@@ -13,7 +13,7 @@ defmodule Application do
 
   Developers typically interact with the application environment and its
   callback module. Therefore those will be the topics we will cover first
-  before jumping into details about the application resource file and life-cycle.
+  before jumping into details about the application resource file and life cycle.
 
   ## The application environment
 
@@ -258,8 +258,8 @@ defmodule Application do
       Application.stop(:ex_unit)
       #=> :ok
 
-  Stopping an application without a callback module is defined, but except for
-  some system tracing, it is in practice a no-op.
+  Stopping an application without a callback module defined, is in practice a
+  no-op, except for some system tracing.
 
   Stopping an application with a callback module has three steps:
 
@@ -277,7 +277,7 @@ defmodule Application do
   invoked only after termination of the whole supervision tree.
 
   Shutting down a live system cleanly can be done by calling `System.stop/1`. It
-  will shut down every application in the opposite order they had been started.
+  will shut down every application in the reverse order they were started.
 
   By default, a SIGTERM from the operating system will automatically translate to
   `System.stop/0`. You can also have more explicit control over operating system
@@ -817,16 +817,29 @@ defmodule Application do
   end
 
   @doc """
-  Puts the environment for multiple apps at the same time.
+  Puts the environment for multiple applications at the same time.
 
   The given config should not:
 
     * have the same application listed more than once
     * have the same key inside the same application listed more than once
 
-  If those conditions are not met, it will raise.
+  If those conditions are not met, this function will raise.
+
+  This function receives the same options as `put_env/4`. Returns `:ok`.
+
+  ## Examples
+
+      Application.put_all_env(
+        my_app: [
+          key: :value,
+          another_key: :another_value
+        ],
+        another_app: [
+          key: :value
+        ]
+      )
 
-  It receives the same options as `put_env/4`. Returns `:ok`.
   """
   @doc since: "1.9.0"
   @spec put_all_env([{app, [{key, value}]}], timeout: timeout, persistent: boolean) :: :ok
@@ -850,7 +863,10 @@ defmodule Application do
 
   # TODO: Remove this deprecation warning on 2.0+ and allow list lookups as in compile_env.
   defp maybe_warn_on_app_env_key(app, key) do
-    message = "passing non-atom as application env key is deprecated, got: #{inspect(key)}"
+    message = fn ->
+      "passing non-atom as application env key is deprecated, got: #{inspect(key)}"
+    end
+
     IO.warn_once({Application, :key, app, key}, message, _stacktrace_drop_levels = 2)
   end
 
@@ -889,11 +905,12 @@ defmodule Application do
 
   ## Options
 
-    * `:type` - if the application should be started in `:permanent`,
-      `:temporary`, or `:transient`. See `t:restart_type/1` for more information.
+    * `:type` - if the application should be started `:temporary` (default),
+      `:permanent`, or `:transient`. See `t:restart_type/1` for more information.
 
     * `:mode` - (since v1.15.0) if the applications should be started serially
-      or concurrently. This option requires Erlang/OTP 26+.
+      (`:serial`, default) or concurrently (`:concurrent`). This option requires
+      Erlang/OTP 26+.
 
   """
   @spec ensure_all_started(app | [app], type: restart_type(), mode: :serial | :concurrent) ::
