@@ -1,11 +1,18 @@
 defmodule Exception do
   @moduledoc """
-  Functions to format throw/catch/exit and exceptions.
+  Functions for dealing with throw/catch/exit and exceptions.
 
-  Note that stacktraces in Elixir are only available inside
-  catch and rescue by using the `__STACKTRACE__/0` variable.
+  This module also defines the behaviour required by custom
+  exceptions. To define your own, see `defexception/1`.
 
-  Do not rely on the particular format returned by the `format*`
+  ## Formatting functions
+
+  Several functions in this module help format exceptions.
+  Some of these functions expect the stacktrace as argument.
+  The stacktrace is typically available inside catch and
+  rescue by using the `__STACKTRACE__/0` variable.
+
+  Do not rely on the particular format returned by the
   functions in this module. They may be changed in future releases
   in order to better suit Elixir's tool chain. In other words,
   by using the functions in this module it is guaranteed you will
@@ -1041,44 +1048,6 @@ defmodule ArgumentError do
   """
 
   defexception message: "argument error"
-
-  @impl true
-  def blame(
-        exception,
-        [{:erlang, :apply, [module, function, args], _} | _] = stacktrace
-      ) do
-    message =
-      cond do
-        not proper_list?(args) ->
-          "you attempted to apply a function named #{inspect(function)} on module #{inspect(module)} " <>
-            "with arguments #{inspect(args)}. Arguments (the third argument of apply) must always be a proper list"
-
-        # Note that args may be an empty list even if they were supplied
-        not is_atom(module) and is_atom(function) and args == [] ->
-          "you attempted to apply a function named #{inspect(function)} on #{inspect(module)}. " <>
-            "If you are using Kernel.apply/3, make sure the module is an atom. " <>
-            "If you are using the dot syntax, such as module.function(), " <>
-            "make sure the left-hand side of the dot is a module atom"
-
-        not is_atom(module) ->
-          "you attempted to apply a function on #{inspect(module)}. " <>
-            "Modules (the first argument of apply) must always be an atom"
-
-        not is_atom(function) ->
-          "you attempted to apply a function named #{inspect(function)} on module #{inspect(module)}. " <>
-            "However, #{inspect(function)} is not a valid function name. Function names (the second argument " <>
-            "of apply) must always be an atom"
-      end
-
-    {%{exception | message: message}, stacktrace}
-  end
-
-  def blame(exception, stacktrace) do
-    {exception, stacktrace}
-  end
-
-  defp proper_list?(list) when length(list) >= 0, do: true
-  defp proper_list?(_), do: false
 end
 
 defmodule ArithmeticError do
@@ -1243,7 +1212,7 @@ defmodule SyntaxError do
       })
       when not is_nil(snippet) and not is_nil(column) do
     snippet =
-      :elixir_errors.format_snippet({line, column}, file, description, snippet, :error, [], nil)
+      :elixir_errors.format_snippet(:error, {line, column}, file, description, snippet, %{})
 
     format_message(file, line, column, snippet)
   end
@@ -1256,7 +1225,7 @@ defmodule SyntaxError do
         description: description
       }) do
     snippet =
-      :elixir_errors.format_snippet({line, column}, file, description, nil, :error, [], nil)
+      :elixir_errors.format_snippet(:error, {line, column}, file, description, nil, %{})
 
     padded = "   " <> String.replace(snippet, "\n", "\n   ")
     format_message(file, line, column, padded)
@@ -1347,7 +1316,7 @@ defmodule TokenMissingError do
         description: description
       }) do
     snippet =
-      :elixir_errors.format_snippet({line, column}, file, description, snippet, :error, [], nil)
+      :elixir_errors.format_snippet(:error, {line, column}, file, description, snippet, %{})
 
     format_message(file, line, column, snippet)
   end
@@ -2074,10 +2043,17 @@ defmodule KeyError do
   defp message(key, term) do
     message = "key #{inspect(key)} not found"
 
-    if term != nil do
-      message <> " in: #{inspect(term, pretty: true, limit: :infinity)}"
-    else
-      message
+    cond do
+      term == nil ->
+        message
+
+      is_atom(term) and is_atom(key) ->
+        message <>
+          " in: #{inspect(term)} (if instead you want to invoke #{inspect(term)}.#{key}(), " <>
+          "make sure to add parentheses after the function name)"
+
+      true ->
+        message <> " in: #{inspect(term, pretty: true, limit: :infinity)}"
     end
   end
 
@@ -2086,11 +2062,6 @@ defmodule KeyError do
     {exception, stacktrace}
   end
 
-  def blame(exception = %{term: nil}, stacktrace) do
-    message = message(exception.key, exception.term)
-    {%{exception | message: message}, stacktrace}
-  end
-
   def blame(exception, stacktrace) do
     %{term: term, key: key} = exception
     message = message(key, term)
@@ -2307,9 +2278,39 @@ defmodule ErlangError do
 
   @doc false
   def normalize(:badarg, stacktrace) do
-    case error_info(:badarg, stacktrace, "errors were found at the given arguments") do
-      {:ok, reason, details} -> %ArgumentError{message: reason <> details}
-      :error -> %ArgumentError{}
+    case stacktrace do
+      [{:erlang, :apply, [module, function, args], _} | _] when not is_atom(module) ->
+        message =
+          cond do
+            is_map(module) and is_atom(function) and is_map_key(module, function) ->
+              "you attempted to apply a function named #{inspect(function)} on a map/struct. " <>
+                "If you are using Kernel.apply/3, make sure the module is an atom. " <>
+                if is_function(module[function]) do
+                  "If you are trying to invoke an anonymous function in a map/struct, " <>
+                    "add a dot between the function name and the parenthesis: map.#{function}.()"
+                else
+                  "If you are using the dot syntax, ensure there are no parentheses " <>
+                    "after the field name, such as map.#{function}"
+                end
+
+            is_atom(function) and args == [] ->
+              "you attempted to apply a function named #{inspect(function)} on #{inspect(module)}. " <>
+                "If you are using Kernel.apply/3, make sure the module is an atom. " <>
+                "If you are using the dot syntax, such as module.function(), " <>
+                "make sure the left-hand side of the dot is an atom representing a module"
+
+            true ->
+              "you attempted to apply a function on #{inspect(module)}. " <>
+                "Modules (the first argument of apply) must always be an atom"
+          end
+
+        %ArgumentError{message: message}
+
+      _ ->
+        case error_info(:badarg, stacktrace, "errors were found at the given arguments") do
+          {:ok, reason, details} -> %ArgumentError{message: reason <> details}
+          :error -> %ArgumentError{}
+        end
     end
   end
 
