@@ -1138,10 +1138,52 @@ defmodule Stream do
   end
 
   @doc """
-  Creates a stream where each element in the enumerable will
-  be wrapped in a tuple alongside its index.
+  Builds a stream from an index, either starting from offset, or given by function.
 
-  If an `offset` is given, we will index from the given offset instead of from zero.
+  May receive a function or an integer offset.
+
+  If an `offset` is given, it will emit elements from offset.
+
+  If a `function` is given, it will invoke the function with
+  elements from offset.
+
+  ## Examples
+
+      iex> Stream.from_index() |> Enum.take(3)
+      [0, 1, 2]
+
+      iex> Stream.from_index(1) |> Enum.take(3)
+      [1, 2, 3]
+
+      iex> Stream.from_index(fn x -> x * 10 end) |> Enum.take(3)
+      [0, 10, 20]
+
+  """
+  @doc since: "1.17.0"
+  @spec from_index(integer) :: Enumerable.t(integer)
+  @spec from_index((integer -> return_value)) :: Enumerable.t(return_value)
+        when return_value: term
+  def from_index(fun_or_offset \\ 0)
+
+  def from_index(offset) when is_integer(offset) do
+    unfold(offset, &{&1, &1 + 1})
+  end
+
+  def from_index(fun) when is_function(fun) do
+    unfold(0, &{fun.(&1), &1 + 1})
+  end
+
+  @doc """
+  Creates a stream where each element in the enumerable will
+  be wrapped in a tuple alongside its index or according to a given function.
+
+  May receive a function or an integer offset.
+
+  If an `offset` is given, it will index from the given offset instead of from
+  zero.
+
+  If a `function` is given, it will index by invoking the function for each
+  element and index (zero-based) of the enumerable.
 
   ## Examples
 
@@ -1153,12 +1195,24 @@ defmodule Stream do
       iex> Enum.to_list(stream)
       [{1, 3}, {2, 4}, {3, 5}]
 
+      iex> stream = Stream.with_index([1, 2, 3], fn x, index -> x + index end)
+      iex> Enum.to_list(stream)
+      [1, 3, 5]
+
   """
-  @spec with_index(Enumerable.t(), integer) :: Enumerable.t()
-  def with_index(enum, offset \\ 0) when is_integer(offset) do
+  @spec with_index(Enumerable.t(), integer) :: Enumerable.t({element, integer})
+  @spec with_index(Enumerable.t(), (element, index -> return_value)) :: Enumerable.t(return_value)
+        when return_value: term
+  def with_index(enum, fun_or_offset \\ 0)
+
+  def with_index(enum, offset) when is_integer(offset) do
     lazy(enum, offset, fn f1 -> R.with_index(f1) end)
   end
 
+  def with_index(enum, fun) when is_function(fun, 2) do
+    lazy(enum, 0, fn f1 -> R.with_index(fun, f1) end)
+  end
+
   ## Combiners
 
   @doc """
@@ -1200,6 +1254,11 @@ defmodule Stream do
   @doc """
   Zips two enumerables together, lazily.
 
+  Because a list of two-element tuples with atoms as the first
+  tuple element is a keyword list (`Keyword`), zipping a first `Stream`
+  of atoms with a second `Stream` of any kind creates a `Stream`
+  that generates a keyword list.
+
   The zipping finishes as soon as either enumerable completes.
 
   ## Examples
@@ -1208,6 +1267,8 @@ defmodule Stream do
       iex> cycle = Stream.cycle([:a, :b, :c])
       iex> Stream.zip(concat, cycle) |> Enum.to_list()
       [{1, :a}, {2, :b}, {3, :c}, {4, :a}, {5, :b}, {6, :c}]
+      iex> Stream.zip(cycle, concat) |> Enum.to_list()
+      [a: 1, b: 2, c: 3, a: 4, b: 5, c: 6]
 
   """
   @spec zip(Enumerable.t(), Enumerable.t()) :: Enumerable.t()
@@ -1511,13 +1572,15 @@ defmodule Stream do
   end
 
   @doc """
-  Emits a sequence of values, starting with `start_value`. Successive
-  values are generated by calling `next_fun` on the previous value.
+  Emits a sequence of values, starting with `start_value`.
+
+  Successive values are generated by calling `next_fun`
+  on the previous value.
 
   ## Examples
 
-      iex> Stream.iterate(0, &(&1 + 1)) |> Enum.take(5)
-      [0, 1, 2, 3, 4]
+      iex> Stream.iterate(1, &(&1 * 2)) |> Enum.take(5)
+      [1, 2, 4, 8, 16]
 
   """
   @spec iterate(element, (element -> element)) :: Enumerable.t()
