@@ -5,6 +5,9 @@ defmodule URI do
   This module provides functions for working with URIs (for example, parsing
   URIs or encoding query strings). The functions in this module are implemented
   according to [RFC 3986](https://tools.ietf.org/html/rfc3986).
+
+  Additionally, the Erlang [`:uri_string` module](`:uri_string`) provides certain functionalities,
+  such as RFC 3986 compliant URI normalization.
   """
 
   @doc """
@@ -235,7 +238,7 @@ defmodule URI do
 
       {{key, value}, rest} ->
         # Avoid warnings about Dict being deprecated
-        dict_module = Dict
+        dict_module = String.to_atom("Dict")
         decode_query_into_dict(rest, dict_module.put(dict, key, value), encoding)
     end
   end
@@ -365,9 +368,9 @@ defmodule URI do
   Percent-encodes all characters that require escaping in `string`.
 
   By default, this function is meant to escape the whole URI, and
-  therefore it will escape all characters which are foreign to the
-  URI specification. Reserved characters (such as `:` and `/`) or
-  unreserved (such as letters and numbers) are not escaped.
+  therefore it will only escape characters which are foreign in
+  all parts of a URI. Reserved characters (such as `:` and `/`)
+  or unreserved (such as letters and numbers) are not escaped.
 
   Because different components of a URI require different escaping
   rules, this function also accepts a `predicate` function as an optional
