@@ -35,6 +35,7 @@ canonical = System.fetch_env!("CANONICAL")
     "lib/elixir/pages/anti-patterns/process-anti-patterns.md",
     "lib/elixir/pages/anti-patterns/macro-anti-patterns.md",
     "lib/elixir/pages/references/compatibility-and-deprecations.md",
+    "lib/elixir/pages/references/gradual-set-theoretic-types.md",
     "lib/elixir/pages/references/library-guidelines.md",
     "lib/elixir/pages/references/naming-conventions.md",
     "lib/elixir/pages/references/operators.md",
@@ -79,6 +80,10 @@ canonical = System.fetch_env!("CANONICAL")
   skip_undefined_reference_warnings_on: [
     "lib/elixir/pages/references/compatibility-and-deprecations.md"
   ],
+  skip_code_autolink_to: [
+    "Enumerable.List",
+    "Inspect.MapSet"
+  ],
   formatters: ["html", "epub"],
   groups_for_modules: [
     # [Kernel, Kernel.SpecialForms],
@@ -89,6 +94,7 @@ canonical = System.fetch_env!("CANONICAL")
       Bitwise,
       Date,
       DateTime,
+      Duration,
       Exception,
       Float,
       Function,
