@@ -1113,163 +1113,165 @@ defmodule CodeFragmentTest do
   end
 
   describe "container_cursor_to_quoted/2" do
-    def s2q(arg, opts \\ []), do: Code.string_to_quoted(arg, opts)
-    def cc2q(arg, opts \\ []), do: CF.container_cursor_to_quoted(arg, opts)
+    def s2q!(arg, opts \\ []), do: Code.string_to_quoted!(arg, opts)
+
+    def cc2q!(arg, opts \\ []) do
+      {:ok, res} = CF.container_cursor_to_quoted(arg, opts)
+      res
+    end
 
     test "completes terminators" do
-      assert cc2q("(") == s2q("(__cursor__())")
-      assert cc2q("[") == s2q("[__cursor__()]")
-      assert cc2q("{") == s2q("{__cursor__()}")
-      assert cc2q("<<") == s2q("<<__cursor__()>>")
-      assert cc2q("foo do") == s2q("foo do __cursor__() end")
-      assert cc2q("foo do true else") == s2q("foo do true else __cursor__() end")
+      assert cc2q!("(") == s2q!("(__cursor__())")
+      assert cc2q!("[") == s2q!("[__cursor__()]")
+      assert cc2q!("{") == s2q!("{__cursor__()}")
+      assert cc2q!("<<") == s2q!("<<__cursor__()>>")
+      assert cc2q!("foo do") == s2q!("foo do __cursor__() end")
+      assert cc2q!("foo do true else") == s2q!("foo do true else __cursor__() end")
     end
 
     test "inside interpolation" do
-      assert cc2q(~S|"foo #{(|) == s2q(~S|"foo #{(__cursor__())}"|)
-      assert cc2q(~S|"foo #{"bar #{{|) == s2q(~S|"foo #{"bar #{{__cursor__()}}"}"|)
+      assert cc2q!(~S|"foo #{(|) == s2q!(~S|"foo #{(__cursor__())}"|)
+      assert cc2q!(~S|"foo #{"bar #{{|) == s2q!(~S|"foo #{"bar #{{__cursor__()}}"}"|)
     end
 
     test "keeps operators" do
-      assert cc2q("1 + 2") == s2q("1 + __cursor__()")
-      assert cc2q("&foo") == s2q("&__cursor__()")
-      assert cc2q("&foo/") == s2q("&foo/__cursor__()")
+      assert cc2q!("1 + 2") == s2q!("1 + __cursor__()")
+      assert cc2q!("&foo") == s2q!("&__cursor__()")
+      assert cc2q!("&foo/") == s2q!("&foo/__cursor__()")
     end
 
     test "keeps function calls without parens" do
-      assert cc2q("alias") == s2q("__cursor__()")
-      assert cc2q("alias ") == s2q("alias __cursor__()")
-      assert cc2q("alias foo") == s2q("alias __cursor__()")
-      assert cc2q("alias Foo.Bar") == s2q("alias __cursor__()")
-      assert cc2q("alias Foo.Bar,") == s2q("alias Foo.Bar, __cursor__()")
-      assert cc2q("alias Foo.Bar, as: ") == s2q("alias Foo.Bar, as: __cursor__()")
+      assert cc2q!("alias") == s2q!("__cursor__()")
+      assert cc2q!("alias ") == s2q!("alias __cursor__()")
+      assert cc2q!("alias foo") == s2q!("alias __cursor__()")
+      assert cc2q!("alias Foo.Bar") == s2q!("alias __cursor__()")
+      assert cc2q!("alias Foo.Bar,") == s2q!("alias Foo.Bar, __cursor__()")
+      assert cc2q!("alias Foo.Bar, as: ") == s2q!("alias Foo.Bar, as: __cursor__()")
     end
 
     test "do-end blocks" do
-      assert cc2q("foo do baz") == s2q("foo do __cursor__() end")
-      assert cc2q("foo do bar; baz") == s2q("foo do bar; __cursor__() end")
-      assert cc2q("foo do bar\nbaz") == s2q("foo do bar\n__cursor__() end")
+      assert cc2q!("foo do baz") == s2q!("foo do __cursor__() end")
+      assert cc2q!("foo do bar; baz") == s2q!("foo do bar; __cursor__() end")
+      assert cc2q!("foo do bar\nbaz") == s2q!("foo do bar\n__cursor__() end")
 
-      assert cc2q("foo(bar do baz") == s2q("foo(bar do __cursor__() end)")
-      assert cc2q("foo(bar do baz ") == s2q("foo(bar do baz(__cursor__()) end)")
-      assert cc2q("foo(bar do baz(") == s2q("foo(bar do baz(__cursor__()) end)")
-      assert cc2q("foo(bar do baz bat,") == s2q("foo(bar do baz(bat, __cursor__()) end)")
-
-      assert {:error, {_, "syntax error before: ", "'end'"}} = cc2q("foo(bar do baz, bat")
+      assert cc2q!("foo(bar do baz") == s2q!("foo(bar do __cursor__() end)")
+      assert cc2q!("foo(bar do baz ") == s2q!("foo(bar do baz(__cursor__()) end)")
+      assert cc2q!("foo(bar do baz(") == s2q!("foo(bar do baz(__cursor__()) end)")
+      assert cc2q!("foo(bar do baz bat,") == s2q!("foo(bar do baz(bat, __cursor__()) end)")
+      assert cc2q!("foo(bar do baz, bat") == s2q!("foo(bar do baz, __cursor__() -> nil end)")
     end
 
     test "keyword lists" do
-      assert cc2q("[bar: ") == s2q("[bar: __cursor__()]")
-      assert cc2q("[bar: baz,") == s2q("[bar: baz, __cursor__()]")
-      assert cc2q("[arg, bar: baz,") == s2q("[arg, bar: baz, __cursor__()]")
-      assert cc2q("[arg: val, bar: baz,") == s2q("[arg: val, bar: baz, __cursor__()]")
+      assert cc2q!("[bar: ") == s2q!("[bar: __cursor__()]")
+      assert cc2q!("[bar: baz,") == s2q!("[bar: baz, __cursor__()]")
+      assert cc2q!("[arg, bar: baz,") == s2q!("[arg, bar: baz, __cursor__()]")
+      assert cc2q!("[arg: val, bar: baz,") == s2q!("[arg: val, bar: baz, __cursor__()]")
 
-      assert cc2q("{arg, bar: ") == s2q("{arg, bar: __cursor__()}")
-      assert cc2q("{arg, bar: baz,") == s2q("{arg, bar: baz, __cursor__()}")
-      assert cc2q("{arg: val, bar: baz,") == s2q("{arg: val, bar: baz, __cursor__()}")
+      assert cc2q!("{arg, bar: ") == s2q!("{arg, bar: __cursor__()}")
+      assert cc2q!("{arg, bar: baz,") == s2q!("{arg, bar: baz, __cursor__()}")
 
-      assert cc2q("foo(bar: ") == s2q("foo(bar: __cursor__())")
-      assert cc2q("foo(bar: baz,") == s2q("foo([bar: baz, __cursor__()])")
-      assert cc2q("foo(arg, bar: ") == s2q("foo(arg, bar: __cursor__())")
-      assert cc2q("foo(arg, bar: baz,") == s2q("foo(arg, [bar: baz, __cursor__()])")
-      assert cc2q("foo(arg: val, bar: ") == s2q("foo(arg: val, bar: __cursor__())")
-      assert cc2q("foo(arg: val, bar: baz,") == s2q("foo([arg: val, bar: baz, __cursor__()])")
+      assert cc2q!("foo(bar: ") == s2q!("foo(bar: __cursor__())")
+      assert cc2q!("foo(bar: baz,") == s2q!("foo([bar: baz, __cursor__()])")
+      assert cc2q!("foo(arg, bar: ") == s2q!("foo(arg, bar: __cursor__())")
+      assert cc2q!("foo(arg, bar: baz,") == s2q!("foo(arg, [bar: baz, __cursor__()])")
+      assert cc2q!("foo(arg: val, bar: ") == s2q!("foo(arg: val, bar: __cursor__())")
+      assert cc2q!("foo(arg: val, bar: baz,") == s2q!("foo([arg: val, bar: baz, __cursor__()])")
 
-      assert cc2q("foo bar: ") == s2q("foo(bar: __cursor__())")
-      assert cc2q("foo bar: baz,") == s2q("foo([bar: baz, __cursor__()])")
-      assert cc2q("foo arg, bar: ") == s2q("foo(arg, bar: __cursor__())")
-      assert cc2q("foo arg, bar: baz,") == s2q("foo(arg, [bar: baz, __cursor__()])")
-      assert cc2q("foo arg: val, bar: ") == s2q("foo(arg: val, bar: __cursor__())")
-      assert cc2q("foo arg: val, bar: baz,") == s2q("foo([arg: val, bar: baz, __cursor__()])")
+      assert cc2q!("foo bar: ") == s2q!("foo(bar: __cursor__())")
+      assert cc2q!("foo bar: baz,") == s2q!("foo([bar: baz, __cursor__()])")
+      assert cc2q!("foo arg, bar: ") == s2q!("foo(arg, bar: __cursor__())")
+      assert cc2q!("foo arg, bar: baz,") == s2q!("foo(arg, [bar: baz, __cursor__()])")
+      assert cc2q!("foo arg: val, bar: ") == s2q!("foo(arg: val, bar: __cursor__())")
+      assert cc2q!("foo arg: val, bar: baz,") == s2q!("foo([arg: val, bar: baz, __cursor__()])")
     end
 
     test "maps and structs" do
-      assert cc2q("%") == s2q("__cursor__()")
-      assert cc2q("%{") == s2q("%{__cursor__()}")
-      assert cc2q("%{bar:") == s2q("%{__cursor__()}")
-      assert cc2q("%{bar: ") == s2q("%{bar: __cursor__()}")
-      assert cc2q("%{bar: baz,") == s2q("%{bar: baz, __cursor__()}")
-      assert cc2q("%{foo | ") == s2q("%{foo | __cursor__()}")
-      assert cc2q("%{foo | bar:") == s2q("%{foo | __cursor__()}")
-      assert cc2q("%{foo | bar: ") == s2q("%{foo | bar: __cursor__()}")
-      assert cc2q("%{foo | bar: baz,") == s2q("%{foo | bar: baz, __cursor__()}")
+      assert cc2q!("%") == s2q!("__cursor__()")
+      assert cc2q!("%{") == s2q!("%{__cursor__()}")
+      assert cc2q!("%{bar:") == s2q!("%{__cursor__()}")
+      assert cc2q!("%{bar: ") == s2q!("%{bar: __cursor__()}")
+      assert cc2q!("%{bar: baz,") == s2q!("%{bar: baz, __cursor__()}")
+      assert cc2q!("%{foo | ") == s2q!("%{foo | __cursor__()}")
+      assert cc2q!("%{foo | bar:") == s2q!("%{foo | __cursor__()}")
+      assert cc2q!("%{foo | bar: ") == s2q!("%{foo | bar: __cursor__()}")
+      assert cc2q!("%{foo | bar: baz,") == s2q!("%{foo | bar: baz, __cursor__()}")
 
-      assert cc2q("%Foo") == s2q("__cursor__()")
-      assert cc2q("%Foo{") == s2q("%Foo{__cursor__()}")
-      assert cc2q("%Foo{bar: ") == s2q("%Foo{bar: __cursor__()}")
-      assert cc2q("%Foo{bar: baz,") == s2q("%Foo{bar: baz, __cursor__()}")
-      assert cc2q("%Foo{foo | ") == s2q("%Foo{foo | __cursor__()}")
-      assert cc2q("%Foo{foo | bar:") == s2q("%Foo{foo | __cursor__()}")
-      assert cc2q("%Foo{foo | bar: ") == s2q("%Foo{foo | bar: __cursor__()}")
-      assert cc2q("%Foo{foo | bar: baz,") == s2q("%Foo{foo | bar: baz, __cursor__()}")
+      assert cc2q!("%Foo") == s2q!("__cursor__()")
+      assert cc2q!("%Foo{") == s2q!("%Foo{__cursor__()}")
+      assert cc2q!("%Foo{bar: ") == s2q!("%Foo{bar: __cursor__()}")
+      assert cc2q!("%Foo{bar: baz,") == s2q!("%Foo{bar: baz, __cursor__()}")
+      assert cc2q!("%Foo{foo | ") == s2q!("%Foo{foo | __cursor__()}")
+      assert cc2q!("%Foo{foo | bar:") == s2q!("%Foo{foo | __cursor__()}")
+      assert cc2q!("%Foo{foo | bar: ") == s2q!("%Foo{foo | bar: __cursor__()}")
+      assert cc2q!("%Foo{foo | bar: baz,") == s2q!("%Foo{foo | bar: baz, __cursor__()}")
     end
 
     test "binaries" do
-      assert cc2q("<<") == s2q("<<__cursor__()>>")
-      assert cc2q("<<foo") == s2q("<<__cursor__()>>")
-      assert cc2q("<<foo, bar") == s2q("<<foo, __cursor__()>>")
-      assert cc2q("<<foo, bar::baz") == s2q("<<foo, bar::__cursor__()>>")
+      assert cc2q!("<<") == s2q!("<<__cursor__()>>")
+      assert cc2q!("<<foo") == s2q!("<<__cursor__()>>")
+      assert cc2q!("<<foo, bar") == s2q!("<<foo, __cursor__()>>")
+      assert cc2q!("<<foo, bar::baz") == s2q!("<<foo, bar::__cursor__()>>")
     end
 
     test "removes tokens until opening" do
-      assert cc2q("(123") == s2q("(__cursor__())")
-      assert cc2q("[foo") == s2q("[__cursor__()]")
-      assert cc2q("{'foo'") == s2q("{__cursor__()}")
-      assert cc2q("foo do :atom") == s2q("foo do __cursor__() end")
-      assert cc2q("foo(:atom") == s2q("foo(__cursor__())")
+      assert cc2q!("(123") == s2q!("(__cursor__())")
+      assert cc2q!("[foo") == s2q!("[__cursor__()]")
+      assert cc2q!("{'foo'") == s2q!("{__cursor__()}")
+      assert cc2q!("foo do :atom") == s2q!("foo do __cursor__() end")
+      assert cc2q!("foo(:atom") == s2q!("foo(__cursor__())")
     end
 
     test "removes tokens until comma" do
-      assert cc2q("[bar, 123") == s2q("[bar, __cursor__()]")
-      assert cc2q("{bar, 'foo'") == s2q("{bar, __cursor__()}")
-      assert cc2q("<<bar, \"sample\"") == s2q("<<bar, __cursor__()>>")
-      assert cc2q("foo(bar, :atom") == s2q("foo(bar, __cursor__())")
-      assert cc2q("foo bar, :atom") == s2q("foo(bar, __cursor__())")
+      assert cc2q!("[bar, 123") == s2q!("[bar, __cursor__()]")
+      assert cc2q!("{bar, 'foo'") == s2q!("{bar, __cursor__()}")
+      assert cc2q!("<<bar, \"sample\"") == s2q!("<<bar, __cursor__()>>")
+      assert cc2q!("foo(bar, :atom") == s2q!("foo(bar, __cursor__())")
+      assert cc2q!("foo bar, :atom") == s2q!("foo(bar, __cursor__())")
     end
 
     test "removes anonymous functions" do
-      assert cc2q("(fn") == s2q("(__cursor__())")
-      assert cc2q("(fn x") == s2q("(__cursor__())")
-      assert cc2q("(fn x ->") == s2q("(__cursor__())")
-      assert cc2q("(fn x -> x") == s2q("(__cursor__())")
-      assert cc2q("(fn x, y -> x + y") == s2q("(__cursor__())")
-      assert cc2q("(fn x, y -> x + y end") == s2q("(__cursor__())")
+      assert cc2q!("(fn") == s2q!("(fn __cursor__() -> nil end)")
+      assert cc2q!("(fn x") == s2q!("(fn __cursor__() -> nil end)")
+      assert cc2q!("(fn x ->") == s2q!("(fn x -> __cursor__() end)")
+      assert cc2q!("(fn x -> x") == s2q!("(fn x -> __cursor__() end)")
+      assert cc2q!("(fn x, y -> x + y") == s2q!("(fn x, y -> x + __cursor__() end)")
+      assert cc2q!("(fn x, y -> x + y end") == s2q!("(__cursor__())")
     end
 
     test "removes closed terminators" do
-      assert cc2q("foo([1, 2, 3]") == s2q("foo(__cursor__())")
-      assert cc2q("foo({1, 2, 3}") == s2q("foo(__cursor__())")
-      assert cc2q("foo((1, 2, 3)") == s2q("foo(__cursor__())")
-      assert cc2q("foo(<<1, 2, 3>>") == s2q("foo(__cursor__())")
-      assert cc2q("foo(bar do :done end") == s2q("foo(__cursor__())")
+      assert cc2q!("foo([1, 2, 3]") == s2q!("foo(__cursor__())")
+      assert cc2q!("foo({1, 2, 3}") == s2q!("foo(__cursor__())")
+      assert cc2q!("foo((1, 2, 3)") == s2q!("foo(__cursor__())")
+      assert cc2q!("foo(<<1, 2, 3>>") == s2q!("foo(__cursor__())")
+      assert cc2q!("foo(bar do :done end") == s2q!("foo(__cursor__())")
     end
 
     test "incomplete expressions" do
-      assert cc2q("foo(123, :") == s2q("foo(123, __cursor__())")
-      assert cc2q("foo(123, %") == s2q("foo(123, __cursor__())")
-      assert cc2q("foo(123, 0x") == s2q("foo(123, __cursor__())")
-      assert cc2q("foo(123, ~") == s2q("foo(123, __cursor__())")
-      assert cc2q("foo(123, ~r") == s2q("foo(123, __cursor__())")
-      assert cc2q("foo(123, ~r/") == s2q("foo(123, __cursor__())")
+      assert cc2q!("foo(123, :") == s2q!("foo(123, __cursor__())")
+      assert cc2q!("foo(123, %") == s2q!("foo(123, __cursor__())")
+      assert cc2q!("foo(123, 0x") == s2q!("foo(123, __cursor__())")
+      assert cc2q!("foo(123, ~") == s2q!("foo(123, __cursor__())")
+      assert cc2q!("foo(123, ~r") == s2q!("foo(123, __cursor__())")
+      assert cc2q!("foo(123, ~r/") == s2q!("foo(123, __cursor__())")
     end
 
     test "no warnings" do
-      assert cc2q(~s"?\\ ") == s2q("__cursor__()")
-      assert cc2q(~s"{fn -> end, ") == s2q("{fn -> nil end, __cursor__()}")
+      assert cc2q!(~s"?\\ ") == s2q!("__cursor__()")
+      assert cc2q!(~s"{fn -> end, ") == s2q!("{fn -> nil end, __cursor__()}")
     end
 
     test "options" do
       opts = [columns: true]
-      assert cc2q("foo(", opts) == s2q("foo(__cursor__())", opts)
-      assert cc2q("foo(123,", opts) == s2q("foo(123,__cursor__())", opts)
+      assert cc2q!("foo(", opts) == s2q!("foo(__cursor__())", opts)
+      assert cc2q!("foo(123,", opts) == s2q!("foo(123,__cursor__())", opts)
 
       opts = [token_metadata: true]
-      assert cc2q("foo(", opts) == s2q("foo(__cursor__())", opts)
-      assert cc2q("foo(123,", opts) == s2q("foo(123,__cursor__())", opts)
+      assert cc2q!("foo(", opts) == s2q!("foo(__cursor__())", opts)
+      assert cc2q!("foo(123,", opts) == s2q!("foo(123,__cursor__())", opts)
 
       opts = [literal_encoder: fn ast, _ -> {:ok, {:literal, ast}} end]
-      assert cc2q("foo(", opts) == s2q("foo(__cursor__())", opts)
-      assert cc2q("foo(123,", opts) == s2q("foo({:literal, 123},__cursor__())", [])
+      assert cc2q!("foo(", opts) == s2q!("foo(__cursor__())", opts)
+      assert cc2q!("foo(123,", opts) == s2q!("foo({:literal, 123},__cursor__())", [])
     end
   end
 end
