@@ -189,7 +189,7 @@ defmodule IntegerTest do
   end
 
   test "to_charlist/2" do
-    module = Integer
+    module = String.to_atom("Elixir.Integer")
 
     assert Integer.to_charlist(42) == ~c"42"
     assert Integer.to_charlist(+42) == ~c"42"
