@@ -140,26 +140,6 @@ defmodule Kernel.ExpansionTest do
   end
 
   describe "import" do
-    test "raises on invalid options" do
-      message = ~r"invalid :only option for import, expected a keyword list with integer values"
-
-      assert_compile_error(message, fn ->
-        expand(quote(do: import(Kernel, only: [invalid: nil])))
-      end)
-
-      message = ~r"invalid :except option for import, expected a keyword list with integer values"
-
-      assert_compile_error(message, fn ->
-        expand(quote(do: import(Kernel, except: [invalid: nil])))
-      end)
-
-      message = ~r/invalid options for import, expected a keyword list, got: "invalid_options"/
-
-      assert_compile_error(message, fn ->
-        expand(quote(do: import(Kernel, "invalid_options")))
-      end)
-    end
-
     test "raises on conflicting options" do
       message =
         ~r":only and :except can only be given together to import when :only is :functions, :macros, or :sigils"
@@ -434,7 +414,7 @@ defmodule Kernel.ExpansionTest do
 
     test "in matches" do
       assert_compile_error(
-        ~r"cannot find or invoke local foo/1 inside match. .+ Called as: foo\(:bar\)",
+        ~r"cannot find or invoke local foo/1 inside a match. .+ Called as: foo\(:bar\)",
         fn ->
           expand(quote(do: foo(:bar) = :bar))
         end
@@ -466,6 +446,11 @@ defmodule Kernel.ExpansionTest do
 
       assert expand(before_expansion) == after_expansion
     end
+
+    test "invalid metadata" do
+      assert expand({:foo, [imports: 2, context: :unknown], [1, 2]}) ==
+               {:foo, [imports: 2, context: :unknown], [1, 2]}
+    end
   end
 
   describe "floats" do
@@ -608,17 +593,25 @@ defmodule Kernel.ExpansionTest do
       assert expand(quote(do: %x{} = 1)) == quote(do: %x{} = 1)
     end
 
-    test "unknown ^keys in structs" do
-      message = ~r"unknown key \^my_key for struct Kernel\.ExpansionTest\.User"
+    test "invalid keys in structs" do
+      assert_compile_error(~r"invalid key :erlang\.\+\(1, 2\) for struct", fn ->
+        expand(
+          quote do
+            %User{(1 + 2) => :my_value}
+          end
+        )
+      end)
+    end
+
+    test "unknown key in structs" do
+      message = ~r"unknown key :foo for struct Kernel\.ExpansionTest\.User"
 
       assert_compile_error(message, fn ->
-        code =
+        expand(
           quote do
-            my_key = :my_key
-            %User{^my_key => :my_value} = %{}
+            %User{foo: :my_value} = %{}
           end
-
-        expand(code)
+        )
       end)
     end
   end
@@ -737,13 +730,13 @@ defmodule Kernel.ExpansionTest do
 
     test "in guards" do
       message =
-        ~r"cannot invoke remote function Hello.something_that_does_not_exist/1 inside guard"
+        ~r"cannot invoke remote function Hello.something_that_does_not_exist/1 inside a guard"
 
       assert_compile_error(message, fn ->
         expand(quote(do: fn arg when Hello.something_that_does_not_exist(arg) -> arg end))
       end)
 
-      message = ~r"cannot invoke remote function :erlang.make_ref/0 inside guard"
+      message = ~r"cannot invoke remote function :erlang.make_ref/0 inside a guard"
 
       assert_compile_error(message, fn ->
         expand(quote(do: fn arg when make_ref() -> arg end))
@@ -751,7 +744,7 @@ defmodule Kernel.ExpansionTest do
     end
 
     test "in guards with bitstrings" do
-      message = ~r"cannot invoke remote function String.Chars.to_string/1 inside guards"
+      message = ~r"cannot invoke remote function String.Chars.to_string/1 inside a guard"
 
       assert_compile_error(message, fn ->
         expand(quote(do: fn arg when "#{arg}foo" == "argfoo" -> arg end))
@@ -1140,6 +1133,12 @@ defmodule Kernel.ExpansionTest do
       assert expand(quote(do: &unknown(&1, &2))) == {:&, [], [{:/, [], [{:unknown, [], nil}, 2]}]}
     end
 
+    test "keeps position meta on & variables" do
+      assert expand(Code.string_to_quoted!("& &1")) |> clean_meta([:counter]) ==
+               {:fn, [{:line, 1}],
+                [{:->, [{:line, 1}], [[{:capture, [line: 1], nil}], {:capture, [line: 1], nil}]}]}
+    end
+
     test "expands remotes" do
       assert expand(quote(do: &List.flatten/2)) ==
                quote(do: &:"Elixir.List".flatten/2)
@@ -1162,7 +1161,7 @@ defmodule Kernel.ExpansionTest do
           fn -> 17 end
         end
 
-      assert clean_meta(expand(before_expansion), [:import, :context, :no_parens]) ==
+      assert clean_meta(expand(before_expansion), [:imports, :context, :no_parens]) ==
                after_expansion
     end
 
@@ -2731,7 +2730,7 @@ defmodule Kernel.ExpansionTest do
       end)
 
       assert_compile_error(
-        ~r"cannot find or invoke local foo/0 inside bitstring size specifier",
+        ~r"cannot find or invoke local foo/0 inside a bitstring size specifier",
         fn ->
           code =
             quote do
@@ -2742,7 +2741,7 @@ defmodule Kernel.ExpansionTest do
         end
       )
 
-      message = ~r"anonymous call is not allowed in bitstring size specifier"
+      message = ~r"anonymous call is not allowed inside a bitstring size specifier"
 
       assert_compile_error(message, fn ->
         code =
@@ -2753,7 +2752,7 @@ defmodule Kernel.ExpansionTest do
         expand(code, [])
       end)
 
-      message = ~r"cannot invoke remote function in bitstring size specifier"
+      message = ~r"cannot invoke remote function inside a bitstring size specifier"
 
       assert_compile_error(message, fn ->
         code =
@@ -2765,7 +2764,7 @@ defmodule Kernel.ExpansionTest do
         expand(code, [])
       end)
 
-      message = ~r"cannot invoke remote function Foo.bar/0 inside bitstring size specifier"
+      message = ~r"cannot invoke remote function Foo.bar/0 inside a bitstring size specifier"
 
       assert_compile_error(message, fn ->
         code =
