@@ -62,7 +62,7 @@ defmodule Kernel.WarningTest do
   end
 
   defmacro will_warn do
-    quote file: "demo" do
+    quote file: "demo", line: true do
       %{dup: 1, dup: 2}
     end
   end
@@ -196,10 +196,10 @@ defmodule Kernel.WarningTest do
   test "operators formed by many of the same character followed by that character" do
     assert_warn_eval(
       [
-        "nofile:1:11",
-        "found \"...\" followed by \".\", please use parens around \"...\" instead"
+        "nofile:1:12",
+        "found \"+++\" followed by \"+\", please use a space between \"+++\" and the next \"+\""
       ],
-      "quote do: ....()"
+      "quote do: 1++++1"
     )
   end
 
@@ -463,17 +463,6 @@ defmodule Kernel.WarningTest do
       fn x -> match?(x, :value) end
       """
     )
-
-    assert_warn_eval(
-      [
-        "nofile:1",
-        "variable \"&1\" is unused (this might happen when using a capture argument as a pattern)",
-        "variable \"&1\" is unused (this might happen when using a capture argument as a pattern)"
-      ],
-      """
-      &match?(&1, :value)
-      """
-    )
   end
 
   test "useless literal" do
@@ -823,16 +812,26 @@ defmodule Kernel.WarningTest do
 
   test "unused import of any of the functions in :only" do
     assert_warn_compile(
-      ["nofile:2:3", "unused import String"],
+      ["nofile:1:1", "unused import String"],
       """
-      defmodule Sample do
-        import String, only: [upcase: 1, downcase: 1]
-        def a, do: nil
-      end
+      import String, only: [upcase: 1, downcase: 1]
+      """
+    )
+  end
+
+  def with(a, b, c), do: [a, b, c]
+
+  test "import matches special form" do
+    assert_warn_compile(
+      [
+        "nofile:1:1",
+        "cannot import Kernel.WarningTest.with/3 because it conflicts with Elixir special forms"
+      ],
+      """
+      import Kernel.WarningTest, only: [with: 3]
+      :ok = with true <- true, true <- true, do: :ok
       """
     )
-  after
-    purge(Sample)
   end
 
   test "duplicated function on import options" do
@@ -904,11 +903,11 @@ defmodule Kernel.WarningTest do
     assert_warn_eval(
       [
         "key :a will be overridden in map",
-        "nofile:4:11\n",
+        "nofile:4:10\n",
         "key :m will be overridden in map",
-        "nofile:5:11\n",
+        "nofile:5:10\n",
         "key 1 will be overridden in map",
-        "nofile:6:11\n"
+        "nofile:6:10\n"
       ],
       """
       defmodule DuplicateMapKeys do
@@ -1099,6 +1098,10 @@ defmodule Kernel.WarningTest do
     purge(UseSample)
   end
 
+  test "deprecated closing sigil delimiter" do
+    assert_warn_eval(["nofile:1:7", "deprecated"], "~S(foo\\))")
+  end
+
   test "deprecated not left in right" do
     assert_warn_eval(["nofile:1:7", "deprecated"], "not 1 in [1, 2, 3]")
   end
@@ -2102,26 +2105,6 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "struct comparisons" do
-    expressions = [
-      ~s(~N"2018-01-28 12:00:00"),
-      ~s(~T"12:00:00"),
-      ~s(~D"2018-01-28"),
-      "%File.Stat{}"
-    ]
-
-    for op <- [:<, :>, :<=, :>=],
-        expression <- expressions do
-      assert capture_err(fn ->
-               Code.eval_string("x #{op} #{expression}", x: 1)
-             end) =~ "invalid comparison with struct literal #{expression}"
-
-      assert capture_err(fn ->
-               Code.eval_string("#{expression} #{op} x", x: 1)
-             end) =~ "invalid comparison with struct literal #{expression}"
-    end
-  end
-
   test "deprecated GenServer super on callbacks" do
     assert_warn_eval(
       ["nofile:1: ", "calling super for GenServer callback handle_call/3 is deprecated"],
