@@ -367,6 +367,29 @@ defmodule KernelTest do
     assert struct_or_map?(%Macro.Env{}, Macro.Env) == true
   end
 
+  defp non_struct_map?(arg) when is_non_struct_map(arg), do: true
+  defp non_struct_map?(_arg), do: false
+
+  defp non_struct_map_or_struct?(arg) when is_non_struct_map(arg) or is_struct(arg), do: true
+  defp non_struct_map_or_struct?(_arg), do: false
+
+  test "is_non_struct_map/1" do
+    assert is_non_struct_map(%{}) == true
+    assert is_non_struct_map([]) == false
+    assert is_non_struct_map(%Macro.Env{}) == false
+    assert is_non_struct_map(%{__struct__: "foo"}) == true
+    assert non_struct_map?(%Macro.Env{}) == false
+    assert non_struct_map?(%{__struct__: "foo"}) == true
+    assert non_struct_map?([]) == false
+    assert non_struct_map?(%{}) == true
+  end
+
+  test "is_non_struct_map/1 and other match works" do
+    assert non_struct_map_or_struct?(%Macro.Env{}) == true
+    assert non_struct_map_or_struct?(%{}) == true
+    assert non_struct_map_or_struct?(10) == false
+  end
+
   defp exception?(arg) when is_exception(arg), do: true
   defp exception?(_arg), do: false
 
@@ -534,28 +557,6 @@ defmodule KernelTest do
       assert fun_in(17.0) == :none
     end
 
-    def dynamic_in(x, y, z) when x in y..z, do: true
-    def dynamic_in(_x, _y, _z), do: false
-
-    test "in dynamic range function guard" do
-      assert dynamic_in(1, 1, 3)
-      assert dynamic_in(2, 1, 3)
-      assert dynamic_in(3, 1, 3)
-
-      assert dynamic_in(1, 3, 1)
-      assert dynamic_in(2, 3, 1)
-      assert dynamic_in(3, 3, 1)
-
-      refute dynamic_in(0, 1, 3)
-      refute dynamic_in(4, 1, 3)
-      refute dynamic_in(0, 3, 1)
-      refute dynamic_in(4, 3, 1)
-
-      refute dynamic_in(2, 1.0, 3)
-      refute dynamic_in(2, 1, 3.0)
-      refute dynamic_in(2.0, 1, 3)
-    end
-
     def dynamic_step_in(x, y, z, w) when x in y..z//w, do: true
     def dynamic_step_in(_x, _y, _z, _w), do: false
 
@@ -597,7 +598,7 @@ defmodule KernelTest do
       assert case_in(1, 1..3) == true
       assert case_in(2, 1..3) == true
       assert case_in(3, 1..3) == true
-      assert case_in(-3, -1..-3) == true
+      assert case_in(-3, -1..-3//-1) == true
     end
 
     def map_dot(map) when map.field, do: true
@@ -951,6 +952,22 @@ defmodule KernelTest do
       defstruct [:foo, :bar]
     end
 
+    test "get_in/1" do
+      users = %{"john" => %{age: 27}, :meg => %{age: 23}}
+      assert get_in(users["john"][:age]) == 27
+      assert get_in(users["dave"][:age]) == nil
+      assert get_in(users["john"].age) == 27
+      assert get_in(users["dave"].age) == nil
+      assert get_in(users.meg[:age]) == 23
+      assert get_in(users.meg.age) == 23
+
+      is_nil = nil
+      assert get_in(is_nil.age) == nil
+
+      assert_raise KeyError, ~r"key :unknown not found", fn -> get_in(users.unknown) end
+      assert_raise KeyError, ~r"key :unknown not found", fn -> get_in(users.meg.unknown) end
+    end
+
     test "get_in/2" do
       users = %{"john" => %{age: 27}, "meg" => %{age: 23}}
       assert get_in(users, ["john", :age]) == 27
@@ -1506,4 +1523,73 @@ defmodule KernelTest do
       """)
     end
   end
+
+  describe "to_timeout/1" do
+    test "works with keyword lists" do
+      assert to_timeout(hour: 2) == 1000 * 60 * 60 * 2
+      assert to_timeout(minute: 74) == 1000 * 60 * 74
+      assert to_timeout(second: 1293) == 1_293_000
+      assert to_timeout(millisecond: 1_234_123) == 1_234_123
+
+      assert to_timeout(hour: 2, minute: 30) == 1000 * 60 * 60 * 2 + 1000 * 60 * 30
+      assert to_timeout(minute: 30, hour: 2) == 1000 * 60 * 60 * 2 + 1000 * 60 * 30
+      assert to_timeout(minute: 74, second: 30) == 1000 * 60 * 74 + 1000 * 30
+    end
+
+    test "raises on invalid values with keyword lists" do
+      for unit <- [:hour, :minute, :second, :millisecond],
+          value <- [-1, 1.0, :not_an_int] do
+        message =
+          "timeout component #{inspect(unit)} must be a non-negative integer, " <>
+            "got: #{inspect(value)}"
+
+        assert_raise ArgumentError, message, fn -> to_timeout([{unit, value}]) end
+      end
+    end
+
+    test "raises on invalid keys with keyword lists" do
+      message =
+        "timeout component :not_a_unit is not a valid timeout component, valid values are: " <>
+          ":week, :day, :hour, :minute, :second, :millisecond"
+
+      assert_raise ArgumentError, message, fn -> to_timeout(minute: 3, not_a_unit: 1) end
+    end
+
+    test "raises on duplicated components with keyword lists" do
+      assert_raise ArgumentError, "timeout component :minute is duplicated", fn ->
+        to_timeout(minute: 3, hour: 2, minute: 1)
+      end
+    end
+
+    test "works with durations" do
+      assert to_timeout(Duration.new!(hour: 2)) == 1000 * 60 * 60 * 2
+      assert to_timeout(Duration.new!(minute: 74)) == 1000 * 60 * 74
+      assert to_timeout(Duration.new!(second: 1293)) == 1_293_000
+      assert to_timeout(Duration.new!(microsecond: {1_234_123, 4})) == 1_234
+
+      assert to_timeout(Duration.new!(hour: 2, minute: 30)) == 1000 * 60 * 60 * 2 + 1000 * 60 * 30
+      assert to_timeout(Duration.new!(minute: 30, hour: 2)) == 1000 * 60 * 60 * 2 + 1000 * 60 * 30
+      assert to_timeout(Duration.new!(minute: 74, second: 30)) == 1000 * 60 * 74 + 1000 * 30
+    end
+
+    test "raises on durations with non-zero months or days" do
+      message = "duration with a non-zero month cannot be reliably converted to timeouts"
+
+      assert_raise ArgumentError, message, fn ->
+        to_timeout(Duration.new!(month: 3))
+      end
+
+      message = "duration with a non-zero year cannot be reliably converted to timeouts"
+
+      assert_raise ArgumentError, message, fn ->
+        to_timeout(Duration.new!(year: 1))
+      end
+    end
+
+    test "works with timeouts" do
+      assert to_timeout(1_000) == 1_000
+      assert to_timeout(0) == 0
+      assert to_timeout(:infinity) == :infinity
+    end
+  end
 end
