@@ -567,9 +567,10 @@ defmodule ModuleTest do
   end
 
   describe "get_attribute/3" do
-    test "returns a list when the attribute is marked as `accummulate: true`" do
+    test "returns a list when the attribute is marked as `accumulate: true`" do
       in_module do
         Module.register_attribute(__MODULE__, :value, accumulate: true)
+        assert Module.get_attribute(__MODULE__, :value) == []
         Module.put_attribute(__MODULE__, :value, 1)
         assert Module.get_attribute(__MODULE__, :value) == [1]
         Module.put_attribute(__MODULE__, :value, 2)
@@ -587,6 +588,19 @@ defmodule ModuleTest do
       end
     end
 
+    test "returns the value of the attribute if persisted" do
+      in_module do
+        Module.register_attribute(__MODULE__, :value, persist: true)
+        assert Module.get_attribute(__MODULE__, :value, 123) == 123
+        Module.put_attribute(__MODULE__, :value, 1)
+        assert Module.get_attribute(__MODULE__, :value) == 1
+        Module.put_attribute(__MODULE__, :value, 2)
+        assert Module.get_attribute(__MODULE__, :value) == 2
+        Module.delete_attribute(__MODULE__, :value)
+        assert Module.get_attribute(__MODULE__, :value, 123) == 123
+      end
+    end
+
     test "returns the passed default if the attribute does not exist" do
       in_module do
         assert Module.get_attribute(__MODULE__, :attribute, :default) == :default
