@@ -110,7 +110,8 @@ defmodule SystemTest do
         env: %{"foo" => "bar", "baz" => nil},
         arg0: "echo",
         stderr_to_stdout: true,
-        parallelism: true
+        parallelism: true,
+        use_stdio: true
       ]
 
       assert {["hello\r\n"], 0} = System.cmd("cmd", ~w[/c echo hello], opts)
@@ -146,7 +147,8 @@ defmodule SystemTest do
         cd: File.cwd!(),
         env: %{"foo" => "bar", "baz" => nil},
         stderr_to_stdout: true,
-        parallelism: true
+        parallelism: true,
+        use_stdio: true
       ]
 
       assert {["bar\r\n"], 0} = System.shell("echo %foo%", opts)
@@ -167,12 +169,30 @@ defmodule SystemTest do
         env: %{"foo" => "bar", "baz" => nil},
         arg0: "echo",
         stderr_to_stdout: true,
-        parallelism: true
+        parallelism: true,
+        use_stdio: true
       ]
 
       assert {["hello\n"], 0} = System.cmd("echo", ["hello"], opts)
     end
 
+    test "cmd/3 (can't use `use_stdio: false, stderr_to_stdout: true`)" do
+      opts = [
+        into: [],
+        cd: File.cwd!(),
+        env: %{"foo" => "bar", "baz" => nil},
+        arg0: "echo",
+        stderr_to_stdout: true,
+        use_stdio: false
+      ]
+
+      message = ~r"cannot use \"stderr_to_stdout: true\" and \"use_stdio: false\""
+
+      assert_raise ArgumentError, message, fn ->
+        System.cmd("echo", ["hello"], opts)
+      end
+    end
+
     test "cmd/3 by line" do
       assert {["hello", "world"], 0} =
                System.cmd("echo", ["hello\nworld"], into: [], lines: 1024)
@@ -209,6 +229,11 @@ defmodule SystemTest do
       assert {"1\n2\n", 0} = System.shell("x=1; echo $x; echo '2'")
     end
 
+    test "shell/1 with empty string" do
+      assert {"", 0} = System.shell("")
+      assert {"", 0} = System.shell("  ")
+    end
+
     @tag timeout: 1_000
     test "shell/1 returns when command awaits input" do
       assert {"", 0} = System.shell("cat", close_stdin: true)
@@ -223,7 +248,8 @@ defmodule SystemTest do
         into: [],
         cd: File.cwd!(),
         env: %{"foo" => "bar", "baz" => nil},
-        stderr_to_stdout: true
+        stderr_to_stdout: true,
+        use_stdio: true
       ]
 
       assert {["bar\n"], 0} = System.shell("echo $foo", opts)
