@@ -38,7 +38,7 @@ defmodule IEx.Autocomplete do
   @doc """
   The expansion logic.
 
-  Some of the expansion has to be use the current shell
+  Some of the expansion has to use the current shell
   environment, which is found via the broker.
   """
   def expand(code, shell \\ IEx.Broker.shell()) do
@@ -336,7 +336,7 @@ defmodule IEx.Autocomplete do
         container_context_map_fields(pairs, map, hint)
 
       {:struct, alias, pairs} when context == :expr ->
-        map = Map.from_struct(alias.__struct__)
+        map = Map.from_struct(alias.__struct__())
         container_context_map_fields(pairs, map, hint)
 
       :bitstring_modifier ->
