@@ -22,33 +22,44 @@ defmodule IEx.Helpers do
 
   There are many other helpers available, here are some examples:
 
-    * `b/1`            - prints callbacks info and docs for a given module
-    * `c/1`            - compiles a file
-    * `c/2`            - compiles a file and writes bytecode to the given path
-    * `cd/1`           - changes the current directory
-    * `clear/0`        - clears the screen
-    * `exports/1`      - shows all exports (functions + macros) in a module
-    * `flush/0`        - flushes all messages sent to the shell
-    * `h/0`            - prints this help message
-    * `h/1`            - prints help for the given module, function or macro
-    * `i/0`            - prints information about the last value
-    * `i/1`            - prints information about the given term
-    * `ls/0`           - lists the contents of the current directory
-    * `ls/1`           - lists the contents of the specified directory
-    * `open/1`         - opens the source for the given module or function in your editor
-    * `pid/1`          - creates a PID from a string
-    * `pid/3`          - creates a PID with the 3 integer arguments passed
-    * `port/1`         - creates a port from a string
-    * `port/2`         - creates a port with the 2 non-negative integers passed
-    * `pwd/0`          - prints the current working directory
-    * `r/1`            - recompiles the given module's source file
-    * `recompile/0`    - recompiles the current project
-    * `ref/1`          - creates a reference from a string
-    * `ref/4`          - creates a reference with the 4 integer arguments passed
-    * `runtime_info/0` - prints runtime info (versions, memory usage, stats)
-    * `t/1`            - prints the types for the given module or function
-    * `v/0`            - retrieves the last value from the history
-    * `v/1`            - retrieves the nth value from the history
+    * `b/1`             - prints callbacks info and docs for a given module
+    * `c/1`             - compiles a file
+    * `c/2`             - compiles a file and writes bytecode to the given path
+    * `cd/1`            - changes the current directory
+    * `clear/0`         - clears the screen
+    * `exports/1`       - shows all exports (functions + macros) in a module
+    * `flush/0`         - flushes all messages sent to the shell
+    * `h/0`             - prints this help message
+    * `h/1`             - prints help for the given module, function or macro
+    * `i/0`             - prints information about the last value
+    * `i/1`             - prints information about the given term
+    * `ls/0`            - lists the contents of the current directory
+    * `ls/1`            - lists the contents of the specified directory
+    * `open/1`          - opens the source for the given module or function in your editor
+    * `pid/1`           - creates a PID from a string
+    * `pid/3`           - creates a PID with the 3 integer arguments passed
+    * `port/1`          - creates a port from a string
+    * `port/2`          - creates a port with the 2 non-negative integers passed
+    * `pwd/0`           - prints the current working directory
+    * `r/1`             - recompiles the given module's source file
+    * `recompile/0`     - recompiles the current project
+    * `ref/1`           - creates a reference from a string
+    * `ref/4`           - creates a reference with the 4 integer arguments passed
+    * `runtime_info/0`  - prints runtime info (versions, memory usage, stats)
+    * `t/1`             - prints the types for the given module or function
+    * `v/0`             - retrieves the last value from the history
+    * `v/1`             - retrieves the nth value from the history
+
+  There are also several helpers available when debugging, such as:
+
+    * `break!/2`        - sets a breakpoint at `Module.function/arity`
+    * `breaks/0`        - prints all breakpoints to the terminal
+    * `c/0`             - a shortcut for `continue/0`
+    * `continue/0`      - continues execution of the current process
+    * `n/0`             - a shortcut for `next/0`
+    * `next/0`          - goes to the next line of the current breakpoint
+    * `remove_breaks/0` - removes all breakpoints and instrumentation from all modules
+    * `whereami/1`      - prints the current location and stacktrace in a pry session
 
   Help for all of those functions can be consulted directly from
   the command line using the `h/1` helper itself. Try:
@@ -121,7 +132,15 @@ defmodule IEx.Helpers do
         end)
 
       true ->
-        do_recompile(options)
+        project = Mix.Project.get()
+
+        if is_nil(project) or
+             project.__info__(:compile)[:source] == String.to_charlist(Path.absname("mix.exs")) do
+          do_recompile(options)
+        else
+          message = "Cannot recompile because the current working directory changed"
+          IO.puts(IEx.color(:eval_error, message))
+        end
     end
   end
 
@@ -155,13 +174,19 @@ defmodule IEx.Helpers do
 
   It expects a list of files to compile and an optional path to write
   the compiled code to. By default files are in-memory compiled.
-  To write compiled files to the current directory, an empty string
-  can be given.
+  To write compiled files to the current directory, `"."` can be given.
 
   It returns the names of the compiled modules.
 
   If you want to recompile an existing module, check `r/1` instead.
 
+  > #### Remote compilation {: .warning}
+  >
+  > When compiling code, warnings and errors may be printed to standard error.
+  > However, when connecting to a remote node, the standard error output is not
+  > redirected to the client. This means that compilation failures will be written
+  > to the logs or the output terminal of the machine you connect to.
+
   ## Examples
 
   In the example below, we pass a directory to where the `c/2` function will
@@ -1001,6 +1026,14 @@ defmodule IEx.Helpers do
     next()
   end
 
+  @doc """
+  A shortcut for `continue/0`.
+  """
+  @doc since: "1.17.0"
+  def c do
+    continue()
+  end
+
   @doc """
   Sets up a breakpoint in the AST of shape `Module.function/arity`
   with the given number of `stops`.
@@ -1101,7 +1134,7 @@ defmodule IEx.Helpers do
   end
 
   @doc """
-  Sets the number of pending stops in the breakpoint
+  Resets the number of pending stops in the breakpoint
   with the given `id` to zero.
 
   Returns `:ok` if there is such breakpoint ID. `:not_found`
@@ -1115,7 +1148,7 @@ defmodule IEx.Helpers do
   defdelegate reset_break(id), to: IEx.Pry
 
   @doc """
-  Sets the number of pending stops in the given module,
+  Resets the number of pending stops in the given module,
   function and arity to zero.
 
   If the module is not instrumented or if the given function
@@ -1354,6 +1387,7 @@ defmodule IEx.Helpers do
       #PID<0.0.0>
 
   """
+  @spec pid(binary | atom) :: pid()
   def pid("#PID<" <> string) do
     :erlang.list_to_pid(~c"<#{string}")
   end
@@ -1381,6 +1415,7 @@ defmodule IEx.Helpers do
       #PID<0.64.2048>
 
   """
+  @spec pid(non_neg_integer, non_neg_integer, non_neg_integer) :: pid()
   def pid(x, y, z)
       when is_integer(x) and x >= 0 and is_integer(y) and y >= 0 and is_integer(z) and z >= 0 do
     :erlang.list_to_pid(
@@ -1400,6 +1435,7 @@ defmodule IEx.Helpers do
 
   """
   @doc since: "1.8.0"
+  @spec port(binary) :: port()
   def port(string) when is_binary(string) do
     :erlang.list_to_port(~c"#Port<#{string}>")
   end
@@ -1416,6 +1452,7 @@ defmodule IEx.Helpers do
 
   """
   @doc since: "1.8.0"
+  @spec port(non_neg_integer, non_neg_integer) :: port()
   def port(major, minor)
       when is_integer(major) and major >= 0 and is_integer(minor) and minor >= 0 do
     :erlang.list_to_port(
@@ -1433,6 +1470,7 @@ defmodule IEx.Helpers do
 
   """
   @doc since: "1.6.0"
+  @spec ref(binary) :: reference()
   def ref(string) when is_binary(string) do
     :erlang.list_to_ref(~c"#Ref<#{string}>")
   end
@@ -1447,6 +1485,7 @@ defmodule IEx.Helpers do
 
   """
   @doc since: "1.6.0"
+  @spec ref(non_neg_integer, non_neg_integer, non_neg_integer, non_neg_integer) :: reference()
   def ref(w, x, y, z)
       when is_integer(w) and w >= 0 and is_integer(x) and x >= 0 and is_integer(y) and y >= 0 and
              is_integer(z) and z >= 0 do
