@@ -2,10 +2,10 @@ defprotocol IEx.Info do
   @fallback_to_any true
 
   @moduledoc """
-  A protocol to print information in IEx about the given datastructure.
+  A protocol to print information in IEx about the given data structure.
 
   `IEx.Helpers.i/1` uses this protocol to display a term-specific list
-  of of information.
+  of information.
 
   By default, an `Any` implementation will be used which returns
   the `"Data type"`, `"Description"` and `"Reference modules"` sections.
@@ -89,7 +89,7 @@ defimpl IEx.Info, for: Atom do
 
       {^atom, beam, _path} ->
         info = :beam_lib.info(beam)
-        Keyword.fetch(info, :module) == {:ok, atom}
+        is_list(info) and Keyword.fetch(info, :module) == {:ok, atom}
     end
   end
 
