@@ -259,10 +259,11 @@ defmodule IEx.Introspection do
     case Code.ensure_loaded(module) do
       {:module, _} ->
         case Code.fetch_docs(module) do
-          {:docs_v1, _, :erlang, _, _, _, _} = erl_docs ->
+          # TODO remove once we require Erlang/OTP 27+
+          {:docs_v1, _, :erlang, "application/erlang+html", _, _, _} = erl_docs ->
             :shell_docs.render(module, erl_docs) |> IO.puts()
 
-          {:docs_v1, _, _, format, %{} = doc, metadata, _} ->
+          {:docs_v1, _, _, format, doc, metadata, _} when is_map(doc) or doc == :none ->
             print_doc([inspect(module)], [], format, doc, metadata)
 
           {:docs_v1, _, _, _, _, _, _} ->
@@ -378,7 +379,8 @@ defmodule IEx.Introspection do
     spec = get_spec(mod, fun, arity)
 
     cond do
-      language == :erlang ->
+      # TODO remove once we require Erlang/OTP 27+
+      language == :erlang and format == "application/erlang+html" ->
         print_erlang_doc(mod, fun, arity, docs_v1)
         :ok
 
@@ -667,7 +669,8 @@ defmodule IEx.Introspection do
   """
   def t(module) when is_atom(module) do
     case :code.get_doc(module) do
-      {:ok, {:docs_v1, _, :erlang, _, _, _, _} = erl_docs} ->
+      # TODO remove once we require Erlang/OTP 27+
+      {:ok, {:docs_v1, _, :erlang, "application/erlang+html", _, _, _} = erl_docs} ->
         :shell_docs.render_type(module, erl_docs) |> IO.puts()
 
       _ ->
@@ -690,7 +693,8 @@ defmodule IEx.Introspection do
 
   def t({module, type}) when is_atom(module) and is_atom(type) do
     case get_docs(module, [:type]) do
-      {:erlang, _, _, erl_docs} ->
+      # TODO remove once we require Erlang/OTP 27+
+      {:erlang, "application/erlang+html", _, erl_docs} ->
         case :shell_docs.render_type(module, type, erl_docs) do
           {:error, :type_missing} -> types_not_found_or_private("#{inspect(module)}.#{type}")
           iodata -> IO.puts(iodata)
@@ -724,7 +728,8 @@ defmodule IEx.Introspection do
 
   def t({module, type, arity}) when is_atom(module) and is_atom(type) and is_integer(arity) do
     case get_docs(module, [:type]) do
-      {:erlang, _, _, erl_docs} ->
+      # TODO remove once we require Erlang/OTP 27+
+      {:erlang, "application/erlang+html", _, erl_docs} ->
         case :shell_docs.render_type(module, type, arity, erl_docs) do
           {:error, :type_missing} -> types_not_found_or_private("#{inspect(module)}.#{type}")
           chardata -> IO.puts(chardata)
