@@ -341,20 +341,72 @@ defmodule IEx.HelpersTest do
     @tag :erlang_doc
     test "prints Erlang module function specs" do
       captured = capture_io(fn -> h(:timer.sleep() / 1) end)
-      assert captured =~ ":timer.sleep/1"
 
-      # TODO Fix for OTP 27 once specs are available
+      # TODO remove once we require Erlang/OTP 27+
       if System.otp_release() < "27" do
+        assert captured =~ ":timer.sleep/1"
         assert captured =~ "-spec sleep(Time) -> ok when Time :: timeout()."
       else
         assert captured =~ "sleep(Time)"
+        assert captured =~ "@spec sleep(time) :: :ok when time: timeout()"
       end
     end
 
     @tag :erlang_doc
     test "handles non-existing Erlang module function" do
       captured = capture_io(fn -> h(:timer.baz() / 1) end)
-      assert captured =~ "No documentation for :timer.baz was found"
+
+      # TODO remove once we require Erlang/OTP 27+
+      if System.otp_release() < "27" do
+        assert captured =~ "No documentation for :timer.baz was found"
+      else
+        assert captured =~ "No documentation for :timer.baz/1 was found"
+      end
+    end
+
+    @tag :erlang_doc
+    # TODO remove once we require Erlang/OTP 27+
+    @tag skip: System.otp_release() < "27"
+    test "erlang -moduledoc" do
+      filename = "sample.erl"
+
+      code = ~s'''
+      -module(sample).
+      -moduledoc "Module documentation.".
+      '''
+
+      with_file(filename, code, fn ->
+        assert c(filename, ".") == [:sample]
+
+        help = capture_iex("h(:sample)")
+        assert help =~ "Module documentation."
+      end)
+    after
+      cleanup_modules([:sample])
+    end
+
+    @tag :erlang_doc
+    # TODO remove once we require Erlang/OTP 27+
+    @tag skip: System.otp_release() < "27"
+    test "erlang -doc" do
+      filename = "sample.erl"
+
+      code = ~s'''
+      -module(sample).
+      -export([hello/0]).
+
+      -doc "Function documentation.".
+      hello() -> world.
+      '''
+
+      with_file(filename, code, fn ->
+        assert c(filename, ".") == [:sample]
+
+        help = capture_iex("h(:sample.hello)")
+        assert help =~ "Function documentation."
+      end)
+    after
+      cleanup_modules([:sample])
     end
 
     test "prints module documentation" do
@@ -1021,13 +1073,13 @@ defmodule IEx.HelpersTest do
     test "prints all types in Erlang module" do
       captured = capture_io(fn -> t(:queue) end)
 
-      # TODO Fix for OTP 27 once specs are available
+      # TODO remove once we require Erlang/OTP 27+
       if System.otp_release() < "27" do
         assert captured =~ "-type queue() :: queue(_)"
         assert captured =~ "-opaque queue(Item)"
       else
-        assert captured =~ "queue()"
-        assert captured =~ "queue(Item)"
+        assert captured =~ "@type queue() :: queue(_)"
+        assert captured =~ "@opaque queue(item)"
       end
     end
 
@@ -1035,22 +1087,22 @@ defmodule IEx.HelpersTest do
     test "prints single type from Erlang module" do
       captured = capture_io(fn -> t(:erlang.iovec()) end)
 
-      # TODO Fix for OTP 27 once specs are available
+      # TODO remove once we require Erlang/OTP 27+
       if System.otp_release() < "27" do
         assert captured =~ "-type iovec() :: [binary()]"
       else
-        assert captured =~ "iovec()"
+        assert captured =~ "@type iovec() :: [binary()]"
       end
 
       assert captured =~ "A list of binaries."
 
       captured = capture_io(fn -> t(:erlang.iovec() / 0) end)
 
-      # TODO Fix for OTP 27 once specs are available
+      # TODO remove once we require Erlang/OTP 27+
       if System.otp_release() < "27" do
         assert captured =~ "-type iovec() :: [binary()]"
       else
-        assert captured =~ "iovec()"
+        assert captured =~ "@type iovec() :: [binary()]"
       end
 
       assert captured =~ "A list of binaries."
@@ -1075,6 +1127,29 @@ defmodule IEx.HelpersTest do
       assert capture_iex("1\n2\nv(2)") == capture_iex("1\n2\nv(-1)")
       assert capture_iex("1\n2\nv(2)") == capture_iex("1\n2\nv()")
     end
+
+    test "returns proper error when trying to access history out of bounds" do
+      # We evaluate 22 statements as 20 is the current limit for iex history
+      assert capture_iex("""
+             \n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n
+             v(1)
+             """) =~
+               "(RuntimeError) v(1) is out of bounds, the currently preserved history ranges from 3 to 22"
+    end
+
+    test "negative lookup works properly after crashes" do
+      capture =
+        capture_iex("""
+            \n
+            \n
+            \n
+            Process.exit(self(), :some_reason)\n
+            :target_value
+            v(-1)
+        """)
+
+      assert capture |> String.split("\n") |> List.last() == ":target_value"
+    end
   end
 
   describe "flush" do
