@@ -20,6 +20,11 @@ defmodule Logger.TranslatorTest do
       raise "oops"
     end
 
+    def handle_call({:execute, fun}, _, state) do
+      fun.()
+      {:reply, :ok, state}
+    end
+
     def handle_call(:error_on_down, {pid, _}, _) do
       mon = Process.monitor(pid)
       assert_receive {:DOWN, ^mon, _, _, _}
@@ -42,6 +47,11 @@ defmodule Logger.TranslatorTest do
       raise "oops"
     end
 
+    def handle_call({:execute, fun}, state) do
+      fun.()
+      {:ok, :ok, state}
+    end
+
     def handle_info(_msg, state) do
       {:ok, state}
     end
@@ -55,6 +65,63 @@ defmodule Logger.TranslatorTest do
     end
   end
 
+  defmodule MyGenStatem do
+    @behaviour :gen_statem
+
+    @impl true
+    def callback_mode, do: :state_functions
+
+    @impl true
+    def init(args) do
+      {:ok, :started, args}
+    end
+
+    def started(:cast, :error, _) do
+      raise "oops"
+    end
+
+    def started({:call, _}, :exit, _) do
+      exit(:oops)
+    end
+
+    def started({:call, _}, :error, _) do
+      raise "oops"
+    end
+
+    def started({:call, from}, {:execute, fun}, data) do
+      fun.()
+      {:keep_state_and_data, {:reply, from, {:started, data}}}
+    end
+
+    def started({:call, {pid, _}}, :error_on_down, _) do
+      mon = Process.monitor(pid)
+      assert_receive {:DOWN, ^mon, _, _, _}
+      raise "oops"
+    end
+  end
+
+  defmodule MyGenStatemHandleEvent do
+    @behaviour :gen_statem
+
+    @impl true
+    def callback_mode, do: :handle_event_function
+
+    @impl true
+    def init(state) do
+      {:ok, :no_state, state}
+    end
+
+    @impl true
+    def handle_event({:call, _}, :error, :no_state, _data) do
+      raise "oops"
+    end
+
+    @impl :gen_statem
+    def format_status(_opts, [_pdict, _, state]) do
+      state
+    end
+  end
+
   defmodule MyBridge do
     @behaviour :supervisor_bridge
 
@@ -143,6 +210,20 @@ defmodule Logger.TranslatorTest do
     refute Map.has_key?(process_metadata, :registered_name)
   end
 
+  @tag skip: System.otp_release() < "27"
+  test "translates GenServer crashes with process label" do
+    {:ok, pid} = GenServer.start(MyGenServer, :ok)
+    :ok = GenServer.call(pid, {:execute, fn -> Process.set_label({:any, "term"}) end})
+
+    assert capture_log(:info, fn -> catch_exit(GenServer.call(pid, :error)) end) =~ ~r"""
+           \[error\] GenServer #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Process Label: {:any, \"term\"}
+           Last message \(from #PID<\d+\.\d+\.\d+>\): :error
+           """s
+  end
+
   test "translates GenServer crashes with custom inspect options" do
     {:ok, pid} = GenServer.start(MyGenServer, List.duplicate(:ok, 1000))
     Application.put_env(:logger, :translator_inspect_opts, limit: 3)
@@ -280,6 +361,25 @@ defmodule Logger.TranslatorTest do
     assert metadata[:registered_name] == config.test
   end
 
+  @tag skip: System.otp_release() < "27"
+  test "translates :gen_event crashes with process label" do
+    {:ok, pid} = :gen_event.start()
+    :ok = :gen_event.add_handler(pid, MyGenEvent, :ok)
+
+    fun = fn -> Process.set_label({:any, "term"}) end
+    :ok = :gen_event.call(pid, MyGenEvent, {:execute, fun})
+
+    assert capture_log(:info, fn ->
+             :gen_event.call(pid, MyGenEvent, :error)
+           end) =~ ~r"""
+           \[error\] :gen_event handler Logger.TranslatorTest.MyGenEvent installed in #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Process Label: {:any, "term"}
+           Last message: :error
+           """s
+  end
+
   test "translates :gen_event crashes on debug" do
     {:ok, pid} = :gen_event.start()
     :ok = :gen_event.add_handler(pid, MyGenEvent, :ok)
@@ -298,6 +398,206 @@ defmodule Logger.TranslatorTest do
     assert {%RuntimeError{message: "oops"}, [_ | _]} = metadata[:crash_reason]
   end
 
+  test "translates :gen_statem crashes" do
+    {:ok, pid} = :gen_statem.start(MyGenStatem, :ok, [])
+
+    assert capture_log(:info, fn ->
+             catch_exit(:gen_statem.call(pid, :error))
+           end) =~ """
+           [error] :gen_statem #{inspect(pid)} terminating
+           ** (RuntimeError) oops
+           """
+
+    assert_receive {:event, {:string, [":gen_statem " <> _ | _]}, gen_statem_metadata}
+    assert_receive {:event, {:string, ["Process " | _]}, process_metadata}
+
+    assert {%RuntimeError{message: "oops"}, [_ | _]} = gen_statem_metadata[:crash_reason]
+    assert {%RuntimeError{message: "oops"}, [_ | _]} = process_metadata[:crash_reason]
+
+    refute Map.has_key?(gen_statem_metadata, :initial_call)
+    assert process_metadata[:initial_call] == {MyGenStatem, :init, 1}
+
+    refute Map.has_key?(gen_statem_metadata, :registered_name)
+    refute Map.has_key?(process_metadata, :registered_name)
+  end
+
+  test "translates :gen_statem crashes with local name", config do
+    {:ok, pid} = :gen_statem.start({:local, config.test}, MyGenStatem, :ok, [])
+    capture_log(:info, fn -> catch_exit(:gen_statem.call(pid, :error)) end)
+
+    assert_receive {:event, {:string, [":gen_statem " <> _ | _]}, gen_statem_metadata}
+    assert_receive {:event, {:string, ["Process " | _]}, process_metadata}
+
+    assert gen_statem_metadata[:registered_name] == config.test
+    assert process_metadata[:registered_name] == config.test
+  end
+
+  test "translates :gen_statem crashes with global name", config do
+    {:ok, pid} = :gen_statem.start({:global, config.test}, MyGenStatem, :ok, [])
+    capture_log(:info, fn -> catch_exit(:gen_statem.call(pid, :error)) end)
+
+    assert_receive {:event, {:string, [":gen_statem " <> _ | _]}, gen_statem_metadata}
+    assert_receive {:event, {:string, ["Process " | _]}, process_metadata}
+
+    assert gen_statem_metadata[:registered_name] == config.test
+    refute Map.has_key?(process_metadata, :registered_name)
+  end
+
+  @tag skip: System.otp_release() < "27"
+  test "translates :gen_statem crashes with process label" do
+    {:ok, pid} = :gen_statem.start(MyGenStatem, :ok, [])
+
+    {:started, :ok} =
+      :gen_statem.call(pid, {:execute, fn -> Process.set_label({:any, "term"}) end})
+
+    assert capture_log(:info, fn -> catch_exit(:gen_statem.call(pid, :error)) end) =~ ~r"""
+           \[error\] :gen_statem #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Process Label: {:any, \"term\"}
+           Queue: .*
+           Postponed: \[\]
+           """s
+  end
+
+  test "translates :gen_statem crashes with custom inspect options" do
+    {:ok, pid} = :gen_statem.start(MyGenStatem, List.duplicate(:ok, 1000), [])
+    Application.put_env(:logger, :translator_inspect_opts, limit: 3)
+
+    assert capture_log(:debug, fn ->
+             catch_exit(:gen_statem.call(pid, :error))
+           end) =~ """
+           State: {:started, [:ok, ...]}
+           """
+  after
+    Application.put_env(:logger, :translator_inspect_opts, [])
+  end
+
+  test "translates :gen_statem crashes on debug" do
+    {:ok, pid} = :gen_statem.start(MyGenStatem, :ok, [])
+
+    assert capture_log(:debug, fn ->
+             catch_exit(GenServer.call(pid, :error))
+           end) =~ ~r"""
+           \[error\] :gen_statem #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Queue: .*
+           Postponed: \[\]
+           State: {:started, :ok}
+           Callback mode: :state_functions, state_enter: false
+           Client #PID<\d+\.\d+\.\d+> is alive
+           .*
+           """s
+
+    assert_receive {:event, {:string, [[":gen_statem " <> _ | _] | _]}, gen_statem_metadata}
+    assert_receive {:event, {:string, ["Process " | _]}, process_metadata}
+
+    assert {%RuntimeError{message: "oops"}, [_ | _]} = gen_statem_metadata[:crash_reason]
+    assert {%RuntimeError{message: "oops"}, [_ | _]} = process_metadata[:crash_reason]
+  end
+
+  test "translates :gen_statem crashed with named client on debug" do
+    {:ok, pid} = :gen_statem.start(MyGenStatem, :ok, [])
+
+    assert capture_log(:debug, fn ->
+             Process.register(self(), :named_client)
+             catch_exit(:gen_statem.call(pid, :error))
+           end) =~ ~r"""
+           \[error\] :gen_statem #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Queue: .*
+           Postponed: \[\]
+           State: {:started, :ok}
+           Callback mode: :state_functions, state_enter: false
+           Client :named_client is alive
+           .*
+           """s
+  end
+
+  test "translates :gen_statem crashes with dead client on debug" do
+    {:ok, pid} = :gen_statem.start(MyGenStatem, :ok, [])
+
+    assert capture_log(:debug, fn ->
+             mon = Process.monitor(pid)
+
+             spawn_link(fn ->
+               catch_exit(:gen_statem.call(pid, :error_on_down, 0))
+             end)
+
+             assert_receive {:DOWN, ^mon, _, _, _}
+           end) =~ ~r"""
+           \[error\] :gen_statem #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Queue: .*
+           Postponed: \[\]
+           State: {:started, :ok}
+           Callback mode: :state_functions, state_enter: false
+           Client #PID<\d+\.\d+\.\d+> is dead
+           """s
+  end
+
+  test "translates :gen_statem crashes with no client" do
+    {:ok, pid} = :gen_statem.start(MyGenStatem, :ok, [])
+
+    assert capture_log(:debug, fn ->
+             mon = Process.monitor(pid)
+             :gen_statem.cast(pid, :error)
+             assert_receive {:DOWN, ^mon, _, _, _}
+           end) =~ ~r"""
+           \[error\] :gen_statem #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Queue: .*
+           Postponed: \[\]
+           State: {:started, :ok}
+           Callback mode: :state_functions, state_enter: false
+           """s
+  end
+
+  test "translates :gen_statem crashes with no client on debug" do
+    {:ok, pid} = :gen_statem.start(MyGenStatem, :ok, [])
+
+    refute capture_log(:debug, fn ->
+             mon = Process.monitor(pid)
+             :gen_statem.cast(pid, :error)
+             assert_receive {:DOWN, ^mon, _, _, _}
+           end) =~ "Client"
+
+    assert_receive {:event, {:string, [[":gen_statem " <> _ | _] | _]}, _gen_statem_metadata}
+    assert_receive {:event, {:string, ["Process " | _]}, _process_metadata}
+  end
+
+  test "translates :gen_statem crashes when callback_mode is :handle_event_function" do
+    {:ok, pid} = :gen_statem.start(MyGenStatemHandleEvent, :ok, [])
+
+    assert capture_log(:debug, fn ->
+             catch_exit(:gen_statem.call(pid, :error))
+           end) =~ ~r"""
+           \[error\] :gen_statem #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Queue: .*
+           Postponed: \[\]
+           State: :ok
+           Callback mode: .*, state_enter: false
+           """s
+
+    assert_receive {:event, {:string, [[":gen_statem " <> _ | _] | _]}, gen_statem_metadata}
+    assert_receive {:event, {:string, ["Process " | _]}, process_metadata}
+
+    assert {%RuntimeError{message: "oops"}, [_ | _]} = gen_statem_metadata[:crash_reason]
+    assert {%RuntimeError{message: "oops"}, [_ | _]} = process_metadata[:crash_reason]
+
+    refute Map.has_key?(gen_statem_metadata, :initial_call)
+    assert process_metadata[:initial_call] == {MyGenStatemHandleEvent, :init, 1}
+
+    refute Map.has_key?(gen_statem_metadata, :registered_name)
+    refute Map.has_key?(process_metadata, :registered_name)
+  end
+
   test "translates Task crashes" do
     {:ok, pid} = Task.start_link(__MODULE__, :task, [self()])
     parent = self()
@@ -428,6 +728,29 @@ defmodule Logger.TranslatorTest do
     assert {:abnormal, [_ | _]} = task_metadata[:crash_reason]
   end
 
+  test "translates Task crashes with process label" do
+    fun = fn ->
+      Process.set_label({:any, "term"})
+      raise "oops"
+    end
+
+    {:ok, pid} = Task.start_link(__MODULE__, :task, [self(), fun])
+
+    assert capture_log(fn ->
+             ref = Process.monitor(pid)
+             send(pid, :go)
+             receive do: ({:DOWN, ^ref, _, _, _} -> :ok)
+           end) =~
+             ~r"""
+             \[error\] Task #{inspect(pid)} started from #{inspect(self())} terminating
+             \*\* \(RuntimeError\) oops
+             .*
+             Process Label: {:any, \"term\"}
+             Function: &Logger.TranslatorTest.task\/2
+                 Args: \[#PID<\d+\.\d+\.\d+>\, .*]
+             """s
+  end
+
   test "translates application start" do
     assert capture_log(fn ->
              Application.start(:eex)
@@ -530,6 +853,33 @@ defmodule Logger.TranslatorTest do
     assert {%RuntimeError{message: "oops"}, [_ | _]} = process_metadata[:crash_reason]
   end
 
+  @tag skip: System.otp_release() < "27"
+  test "translates :proc_lib crashes with process label" do
+    fun = fn ->
+      Process.set_label({:any, "term"})
+      raise "oops"
+    end
+
+    pid = :proc_lib.spawn_link(__MODULE__, :task, [self(), fun])
+
+    assert capture_log(:info, fn ->
+             ref = Process.monitor(pid)
+             send(pid, :message)
+             send(pid, :go)
+             receive do: ({:DOWN, ^ref, _, _, _} -> :ok)
+           end) =~ ~r"""
+           \[error\] Process #PID<\d+\.\d+\.\d+> terminating
+           \*\* \(RuntimeError\) oops
+           .*
+           Initial Call: Logger.TranslatorTest.task/2
+           Process Label: {:any, \"term\"}
+           Ancestors: \[#PID<\d+\.\d+\.\d+>\]
+           """s
+
+    assert_receive {:event, {:string, ["Process " | _]}, process_metadata}
+    assert {%RuntimeError{message: "oops"}, [_ | _]} = process_metadata[:crash_reason]
+  end
+
   test "translates :proc_lib crashes without initial call" do
     fun = fn ->
       Process.delete(:"$initial_call")
