@@ -3,7 +3,7 @@ defmodule Mix do
   Mix is a build tool that provides tasks for creating, compiling,
   and testing Elixir projects, managing its dependencies, and more.
 
-  ## Mix.Project
+  ## `Mix.Project`
 
   The foundation of Mix is a project. A project can be defined by using
   `Mix.Project` in a module, usually placed in a file named `mix.exs`:
@@ -36,7 +36,7 @@ defmodule Mix do
   The best way to get started with your first project is by calling
   `mix new my_project` from the command line.
 
-  ## Mix.Task
+  ## `Mix.Task`
 
   Tasks are what make Mix extensible.
 
@@ -143,7 +143,7 @@ defmodule Mix do
 
   ## Configuration
 
-  Mix allows you configure the application environment of your application
+  Mix allows you to configure the application environment of your application
   and of your dependencies. See the `Application` module to learn more about
   the application environment. On this section, we will focus on how to configure
   it at two distinct moments: build-time and runtime.
@@ -291,13 +291,13 @@ defmodule Mix do
   then `mix cmd` to execute a command line shell script. This shows how
   powerful aliases mixed with Mix tasks can be.
 
-  One commit pitfall of aliases comes when trying to invoke the same task
+  One common pitfall of aliases comes when trying to invoke the same task
   multiple times. Mix tasks are designed to run only once. This prevents
   the same task from being executed multiple times. For example, if there
   are several tasks depending on `mix compile`, the code will be compiled
   only once.
 
-  Similary, `mix format` can only be invoked once. So if you have an alias
+  Similarly, `mix format` can only be invoked once. So if you have an alias
   that attempts to invoke `mix format` multiple times, it won't work unless
   it is explicitly reenabled using `Mix.Task.reenable/1`:
 
@@ -320,7 +320,7 @@ defmodule Mix do
 
   ## Environment variables
 
-  Several environment variables can be used to modify Mix's behaviour.
+  Several environment variables can be used to modify Mix's behavior.
 
   Mix responds to the following variables:
 
@@ -349,7 +349,7 @@ defmodule Mix do
     * `MIX_HOME` - path to Mix's home directory, stores configuration files and scripts used by Mix
       (default: `~/.mix`)
 
-    * `MIX_INSTALL_DIR` - (since v1.12.0) specifies directory where `Mix.install/2` keeps
+    * `MIX_INSTALL_DIR` *(since v1.12.0)* - specifies directory where `Mix.install/2` keeps
        install cache
 
     * `MIX_PATH` - appends extra code paths
@@ -365,7 +365,7 @@ defmodule Mix do
     * `MIX_TARGET` - specifies which target should be used. See [Targets](#module-targets)
 
     * `MIX_XDG` - asks Mix to follow the [XDG Directory Specification](https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html)
-      for its home directory and configuration files. This behaviour needs to
+      for its home directory and configuration files. This behavior needs to
       be opt-in due to backwards compatibility. `MIX_HOME` has higher preference
       than `MIX_XDG`. If none of the variables are set, the default directory
       `~/.mix` will be used
@@ -374,6 +374,14 @@ defmodule Mix do
   flags) should be set to either `1` or `true`, for example:
 
       $ MIX_DEBUG=1 mix compile
+
+  In addition, Mix also uses the following environment variables defined by other libraries
+
+    * `HEX_CACERTS_PATH` - use specified CA certificate file instead of default
+      system CA certificates. This configures how HTTPS calls are made via
+      [Erlang `ssl` module](https://www.erlang.org/doc/man/ssl#type-client_cafile)
+      to fetch remote archives and packages. For more details, see
+      [`mix hex.config`](https://hexdocs.pm/hex/Mix.Tasks.Hex.Config.html#module-config-keys).
   """
 
   @mix_install_project __MODULE__.InstallProject
@@ -677,23 +685,23 @@ defmodule Mix do
     * `:elixir` - if set, ensures the current Elixir version matches the given
       version requirement (Default: `nil`)
 
-    * `:system_env` (since v1.13.0) - a list or a map of system environment variable
+    * `:system_env` *(since v1.13.0)* - a list or a map of system environment variable
       names with respective values as binaries. The system environment is made part
       of the `Mix.install/2` cache, so different configurations will lead to different apps
 
-    * `:config` (since v1.13.0) - a keyword list of keyword lists of compile-time
+    * `:config` *(since v1.13.0)* - a keyword list of keyword lists of compile-time
       configuration. The configuration is part of the `Mix.install/2` cache, so
       different configurations will lead to different apps. For this reason, you
       want to minimize the amount of configuration set through this option.
       Use `Application.put_all_env/2` for setting other runtime configuration.
 
-    * `:config_path` (since v1.14.0) - path to a configuration file. If a `runtime.exs`
+    * `:config_path` *(since v1.14.0)* - path to a configuration file. If a `runtime.exs`
       file exists in the same directory as the given path, it is loaded too.
 
-    * `:lockfile` (since v1.14.0) - path to a lockfile to be used as a basis of
+    * `:lockfile` *(since v1.14.0)* - path to a lockfile to be used as a basis of
       dependency resolution.
 
-    * `:start_applications` (since v1.15.3) - if `true`, ensures that installed app
+    * `:start_applications` *(since v1.15.3)* - if `true`, ensures that installed app
       and its dependencies are started after install (Default: `true`)
 
   ## Examples
@@ -749,7 +757,7 @@ defmodule Mix do
   ## Limitations
 
   There is one limitation to `Mix.install/2`, which is actually an Elixir
-  behaviour. If you are installing a dependency that defines a struct or
+  behavior. If you are installing a dependency that defines a struct or
   macro, you cannot use the struct or macro immediately after the install
   call. For example, this won't work:
 
@@ -760,7 +768,7 @@ defmodule Mix do
   it executes the code. This means that, by the time Elixir tries to expand
   the `%Decimal{}` struct, the dependency has not been installed yet.
 
-  Luckily this has a straightforward solution, which is move the code to
+  Luckily this has a straightforward solution, which is to move the code
   inside a module:
 
       Mix.install([:decimal])
@@ -780,7 +788,12 @@ defmodule Mix do
   ## Environment variables
 
   The `MIX_INSTALL_DIR` environment variable configures the directory that
-  caches all `Mix.install/2`.
+  caches all `Mix.install/2`. It defaults to the "mix/install" folder in the
+  default user cache of your operating system. You can use `install_project_dir/0`
+  to access the directory of an existing install (alongside other installs):
+
+      iex> Mix.install([])
+      iex> Mix.install_project_dir()
 
   The `MIX_INSTALL_FORCE` is available since Elixir v1.13.0 and forces
   `Mix.install/2` to discard any previously cached entry of the current install.
@@ -1085,6 +1098,7 @@ defmodule Mix do
   Returns whether `Mix.install/2` was called in the current node.
   """
   @doc since: "1.13.0"
+  @spec installed?() :: boolean()
   def installed? do
     safe_get_installed() != nil
   end
