@@ -12,7 +12,7 @@ defmodule Mix.Dep.Loader do
   are included as children.
 
   By default, it will filter all dependencies that does not match
-  current environment, behaviour can be overridden via options.
+  current environment, behavior can be overridden via options.
   """
   def children(locked?) do
     mix_children(Mix.Project.config(), locked?, []) ++ Mix.Dep.Umbrella.unloaded()
@@ -358,7 +358,7 @@ defmodule Mix.Dep.Loader do
   end
 
   defp mix_children(config, locked?, opts) do
-    from = Path.absname("mix.exs")
+    from = Mix.Project.project_file()
 
     (config[:deps] || [])
     |> Enum.map(&to_dep(&1, from, _manager = nil, locked?))
@@ -403,15 +403,7 @@ defmodule Mix.Dep.Loader do
   end
 
   defp recently_fetched?(%Mix.Dep{opts: opts, scm: scm}) do
-    scm.fetchable? &&
-      Mix.Utils.stale?(
-        join_stale(opts, :dest, ".fetch"),
-        join_stale(opts, :build, ".mix/compile.fetch")
-      )
-  end
-
-  defp join_stale(opts, key, file) do
-    [Path.join(opts[key], file)]
+    scm.fetchable?() and not File.exists?(Path.join(opts[:build], ".mix/compile.fetch"))
   end
 
   defp app_status(app_path, app, req) do
