@@ -3,19 +3,21 @@ defmodule Mix.Rebar do
 
   @doc """
   Returns the path supposed to host the local copy of `rebar`.
+
+  The rebar3 installation is specific to the Elixir version,
+  in order to force updates when new Elixir versions come out.
   """
-  def local_rebar_path(manager) do
-    Path.join(Mix.Utils.mix_home(), Atom.to_string(manager))
+  def local_rebar_path(:rebar3) do
+    [major, minor | _] = String.split(System.version(), ".")
+    Path.join([Mix.Utils.mix_home(), "elixir", "#{major}-#{minor}", "rebar3"])
   end
 
   @doc """
-  Returns the path to the global copy of `rebar`, defined by the
-  environment variables `MIX_REBAR` or `MIX_REBAR3`.
+  Returns the path to the global copy of `rebar` defined by the
+  environment variable `MIX_REBAR3`.
   """
-  def global_rebar_cmd(manager) do
-    env = manager_to_env(manager)
-
-    if cmd = System.get_env(env) do
+  def global_rebar_cmd(:rebar3) do
+    if cmd = System.get_env("MIX_REBAR3") do
       wrap_cmd(cmd)
     end
   end
@@ -34,6 +36,12 @@ defmodule Mix.Rebar do
   @doc """
   Returns the path to the available `rebar` command.
   """
+  # TODO: Remove on Elixir v1.20 because phx_new and other installers rely on it.
+  def rebar_cmd(:rebar) do
+    Mix.shell().error("[warning] :rebar is no longer supported in Mix, falling back to :rebar3")
+    rebar_cmd(:rebar3)
+  end
+
   def rebar_cmd(manager) do
     global_rebar_cmd(manager) || local_rebar_cmd(manager)
   end
@@ -102,28 +110,10 @@ defmodule Mix.Rebar do
     end
   end
 
-  @doc """
-  Runs `fun` for the given config and for each `sub_dirs` in the
-  given Rebar config.
-
-  `sub_dirs` is only supported in Rebar 2. In Rebar 3, the equivalent
-  to umbrella apps cannot be used as dependencies, so we don't need
-  to worry about such cases in Mix.
-  """
-  def recur(config, fun) do
-    subs =
-      (config[:sub_dirs] || [])
-      |> Enum.flat_map(&Path.wildcard(&1))
-      |> Enum.filter(&File.dir?(&1))
-      |> Enum.flat_map(&recur(load_config(&1), fun))
-
-    [fun.(config) | subs]
-  end
-
   # Translate a Rebar dependency declaration to a Mix declaration
   # From http://www.rebar3.org/docs/dependencies#section-declaring-dependencies
   defp parse_dep(app) when is_atom(app) do
-    {app, ">= 0.0.0", override: true}
+    {app, override: true}
   end
 
   defp parse_dep({app, req}) when is_list(req) do
@@ -141,7 +131,13 @@ defmodule Mix.Rebar do
   defp parse_dep({app, req, source, opts}) do
     source = parse_source(source)
     compile = if :proplists.get_value(:raw, opts, false), do: [compile: false], else: []
-    {app, compile_req(req), [override: true] ++ source ++ compile}
+    opts = [override: true] ++ source ++ compile
+
+    if req do
+      {app, compile_req(req), opts}
+    else
+      {app, opts}
+    end
   end
 
   defp parse_source({:pkg, pkg}) do
@@ -170,10 +166,6 @@ defmodule Mix.Rebar do
     [{scm, to_string(url)}] ++ ref ++ additional_opts
   end
 
-  defp compile_req(nil) do
-    ">= 0.0.0"
-  end
-
   defp compile_req(req) do
     req = List.to_string(req)
 
@@ -192,9 +184,6 @@ defmodule Mix.Rebar do
     end
   end
 
-  defp manager_to_env(:rebar), do: "MIX_REBAR"
-  defp manager_to_env(:rebar3), do: "MIX_REBAR3"
-
   defp eval_script(script_path, config) do
     script = String.to_charlist(Path.basename(script_path))
 
@@ -229,7 +218,7 @@ defmodule Mix.Rebar do
   defp wrap_cmd(rebar) do
     cond do
       not match?({:win32, _}, :os.type()) ->
-        rebar
+        String.replace(rebar, " ", "\\ ")
 
       String.ends_with?(rebar, ".cmd") ->
         "\"#{String.replace(rebar, "/", "\\")}\""
