@@ -6,7 +6,7 @@ defmodule Mix.Tasks.Profile.Cprof do
   @moduledoc """
   Profiles the given file or expression using Erlang's `cprof` tool.
 
-  `cprof` can be useful when you want to discover the bottlenecks related
+  [`:cprof`](`:cprof`) can be useful when you want to discover the bottlenecks related
   to function calls.
 
   Before running the code, it invokes the `app.start` task which compiles
