@@ -402,8 +402,8 @@ defmodule Mix.Tasks.Release do
 
       If you are setting this option manually, we recommend the cookie option
       to be a long and randomly generated string, such as:
-      `Base.url_encode64(:crypto.strong_rand_bytes(40))`. We also recommend to restrict
-      the characters in the cookie to the subset returned by `Base.url_encode64/1`.
+      `Base.encode32(:crypto.strong_rand_bytes(40))`. We also recommend restricting
+      the characters in the cookie to only alphanumeric characters and underscore.
 
     * `:validate_compile_env` - by default a release will match all runtime
       configuration against any configuration that was marked at compile time
@@ -590,6 +590,21 @@ defmodule Mix.Tasks.Release do
 
   starting the release with `bin/myapp start --foo bar baz` will print `start --foo bar baz`.
 
+  ### `epmd`-less deployment
+
+  When a distributed Erlang/Elixir node starts, it runs a separate daemon called EPMD
+  (Erlang Port Mapper Daemon) and registers the node name within EPMD. It is possible
+  to skip this additional Operating System process by setting the following flags in
+  your vm.args files:
+
+      # In vm.args.eex
+      -start_epmd false -erl_epmd_port 6789
+
+      # In remote.vm.args.eex
+      -start_epmd false -erl_epmd_port 6789 -dist_listen false
+
+  You can pick any port of your choice.
+
   ## Application configuration
 
   Mix provides two mechanisms for configuring the application environment
@@ -757,8 +772,7 @@ defmodule Mix.Tasks.Release do
   We document this structure for completeness. In practice, developers
   should not modify any of those files after the release is assembled.
   Instead use env scripts, custom config provider, overlays, and all
-  other mechanisms described in this guide to configure how your release
-  works.
+  other mechanisms described here to configure how your release works.
 
   ## Environment variables
 
@@ -819,9 +833,9 @@ defmodule Mix.Tasks.Release do
 
     * `RELEASE_DISTRIBUTION` - how do we want to run the distribution.
       May be `name` (long names), `sname` (short names) or `none`
-      (distribution is not started automatically). Defaults to
-      `sname` which allows access only within the current system.
-      `name` allows external connections
+      (distribution is not started automatically). Defaults to `sname`.
+      When connecting nodes across hosts, you typically want to set
+      this to `name` (required to use IPs as host names)
 
     * `RELEASE_BOOT_SCRIPT` - the name of the boot script to use when starting
       the release. This script is used when running commands such as `start` and
@@ -1501,7 +1515,7 @@ defmodule Mix.Tasks.Release do
     end
   end
 
-  embed_template(:vm_args, Mix.Tasks.Release.Init.vm_args_text())
+  embed_template(:vm_args, Mix.Tasks.Release.Init.vm_args_text(false))
   embed_template(:env, Mix.Tasks.Release.Init.env_text())
   embed_template(:cli, Mix.Tasks.Release.Init.cli_text())
   embed_template(:env_bat, Mix.Tasks.Release.Init.env_bat_text())
