@@ -71,11 +71,17 @@ defmodule Mix.Tasks.Xref do
 
   ## mix xref graph
 
-  Prints a file dependency graph where an edge from `A` to `B` indicates
+  Emits a file dependency graph where an edge from `A` to `B` indicates
   that `A` (source) depends on `B` (sink).
 
       $ mix xref graph --format stats
 
+  For any non-small project, the output of `mix xref graph` itself, without
+  any additional flags, is not useful: once your project grows, it is hard
+  to gather actionable feedback by looking at the graph as a whole. Instead,
+  `mix xref graph` is better used as a "database", which can help you answer
+  queries about your project.
+
   The following options are accepted:
 
     * `--exclude` - path to exclude. Can be repeated to exclude multiple paths.
@@ -129,7 +135,7 @@ defmodule Mix.Tasks.Xref do
       * `dot` - produces a DOT graph description in `xref_graph.dot` in the
         current directory. Warning: this will override any previously generated file
 
-    * `--output` (since v1.15.0) - can be set to one of
+    * `--output` *(since v1.15.0)* - can be set to one of
 
       * `-` - prints the output to standard output;
 
@@ -234,10 +240,37 @@ defmodule Mix.Tasks.Xref do
   Elixir tracks three types of dependencies between modules: compile,
   exports, and runtime. If a module has a compile time dependency on
   another module, the caller module has to be recompiled whenever the
-  callee changes. Compile-time dependencies are typically added when
-  using macros or when invoking functions in the module body (outside
-  of functions). You can list all dependencies in a file by running
-  `mix xref trace path/to/file.ex`.
+  callee changes (or any runtime dependency of the callee changes).
+  Let's see an example:
+
+      # lib/a.ex
+      defmodule A do
+        @hello B.hello()
+        def hello, do: @hello
+      end
+
+      # lib/b.ex
+      defmodule B do
+        def hello, do: "hello"
+        def world, do: C.world()
+      end
+
+      # lib/c.ex
+      defmodule C do
+        def world, do: "world"
+      end
+
+  If `C.world/0` changes, `B` is marked as stale. `B` does not need to
+  be recompiled, because it depends on `C` at runtime, but anything that
+  depends on `B` at compile-time has to recompile, and that includes `A`.
+
+  Compile-time dependencies are typically added when using macros or
+  when invoking functions in the module body (outside of functions).
+  You can list all dependencies in a file by running
+  `mix xref trace path/to/file.ex`. This type of transitive compile-time
+  dependencies, such as `A` depending on `C` at compile-time through `B`,
+  can be found with the "compile-connected" label, as in
+  `mix xref graph --label compile-connected`.
 
   Export dependencies are compile time dependencies on the module API,
   namely structs and its public definitions. For example, if you import
@@ -1003,10 +1036,12 @@ defmodule Mix.Tasks.Xref do
   end
 
   defp cycles(graph, opts) do
+    # Vertices order in cyclic_strong_components/1 return is arbitrary and changes between
+    # OTP versions, sorting is necessary to make the output stable across versions.
     cycles =
       graph
       |> :digraph_utils.cyclic_strong_components()
-      |> Enum.reduce([], &inner_cycles(graph, &1, &2))
+      |> Enum.reduce([], &inner_cycles(graph, Enum.sort(&1), &2))
       |> Enum.map(&{length(&1), &1})
 
     if min = opts[:min_cycle_size], do: Enum.filter(cycles, &(elem(&1, 0) > min)), else: cycles
