@@ -655,24 +655,23 @@ defmodule Mix.Utils do
     headers = [{~c"user-agent", ~c"Mix/#{System.version()}"}]
     request = {:binary.bin_to_list(path), headers}
 
-    # Use the system certificates if available, otherwise skip peer verification
-    # TODO: Always use system certificates when OTP >= 25.1 is required
-    ssl_options =
-      if Code.ensure_loaded?(:httpc) and function_exported?(:httpc, :ssl_verify_host_options, 1) do
-        try do
-          apply(:httpc, :ssl_verify_host_options, [true])
-        rescue
-          _ ->
-            Mix.shell().error(
-              "warning: failed to load system certificates. SSL peer verification will be skipped but downloads are still verified with a checksum"
-            )
-
-            [verify: :verify_none]
-        end
-      else
-        [verify: :verify_none]
+    # allow override of system CA certs to support running on managed networks
+    # using an SSL proxy etc. Piggy back on Hex defined environment variable
+    # rather than creating a new one, as these are almost always going to be
+    # set and used together.
+    cacert_opt =
+      case System.get_env("HEX_CACERTS_PATH") do
+        nil -> {:cacerts, :public_key.cacerts_get()}
+        file -> {:cacertfile, file}
       end
 
+    # Use the system certificates
+    ssl_options = [
+      cacert_opt,
+      verify: :verify_peer,
+      customize_hostname_check: [match_fun: :public_key.pkix_verify_hostname_match_fun(:https)]
+    ]
+
     # We are using relaxed: true because some servers is returning a Location
     # header with relative paths, which does not follow the spec. This would
     # cause the request to fail with {:error, :no_scheme} unless :relaxed
