@@ -19,6 +19,46 @@ defmodule Mix.CLITest do
     end)
   end
 
+  test "custom default task" do
+    in_fixture("no_mixfile", fn ->
+      File.write!("mix.exs", """
+      defmodule P do
+        use Mix.Project
+        def cli, do: [default_task: "oops"]
+        def project, do: [app: :p, version: "0.1.0"]
+      end
+      """)
+
+      assert mix(~w[]) =~ "The task \"oops\" could not be found"
+    end)
+  end
+
+  test "Mix.raise/2 can set exit status", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
+      File.mkdir_p!("lib")
+
+      File.write!("mix.exs", """
+      defmodule MyProject do
+        use Mix.Project
+
+        def project do
+          [app: :my_project, version: "0.0.1", aliases: aliases()]
+        end
+
+        defp aliases do
+          [
+            custom: &error(&1, exit_status: 99),
+          ]
+        end
+
+        defp error(_args, opts), do: Mix.raise("oops", opts)
+      end
+      """)
+
+      assert {_, 99} = mix_code(~w[custom])
+    end)
+  end
+
   test "compiles and invokes simple task from CLI", %{tmp_dir: tmp_dir} do
     File.cd!(tmp_dir, fn ->
       File.mkdir_p!("lib")
@@ -60,11 +100,11 @@ defmodule Mix.CLITest do
       assert contents =~ "This won't appear"
 
       contents = mix(~w[my_hello], [{"MIX_DEBUG", "1"}])
-      assert contents =~ "** Running mix my_hello (inside MyProject)"
+      assert contents =~ "-> Running mix my_hello (inside MyProject)"
       assert contents =~ "** (Mix.Error) oops"
 
       contents = mix(~w[my_hello], [{"MIX_DEBUG", "0"}])
-      refute contents =~ "** Running mix my_hello (inside MyProject)"
+      refute contents =~ "-> Running mix my_hello (inside MyProject)"
       refute contents =~ "** (Mix.Error) oops"
     end)
   end
@@ -76,7 +116,7 @@ defmodule Mix.CLITest do
     end)
   end
 
-  test "tasks with slashes in them raise a NoTaskError right away", %{tmp_dir: tmp_dir} do
+  test "tasks with slashes raise NoTaskError", %{tmp_dir: tmp_dir} do
     File.cd!(tmp_dir, fn ->
       contents = mix(~w[my/task])
       assert contents =~ "** (Mix) The task \"my/task\" could not be found"
@@ -98,7 +138,7 @@ defmodule Mix.CLITest do
     end)
   end
 
-  test "env config", %{tmp_dir: tmp_dir} do
+  test "env var config", %{tmp_dir: tmp_dir} do
     File.cd!(tmp_dir, fn ->
       File.write!("custom.exs", """
       defmodule P do
@@ -108,66 +148,95 @@ defmodule Mix.CLITest do
       """)
 
       System.put_env("MIX_ENV", "prod")
+      System.put_env("MIX_TARGET", "rpi")
       System.put_env("MIX_EXS", "custom.exs")
 
-      output = mix(["run", "-e", "IO.inspect {Mix.env(), System.argv()}", "--", "1", "2", "3"])
-      assert output =~ ~s({:prod, ["1", "2", "3"]})
+      inspect = "IO.inspect {Mix.env(), Mix.target(), System.argv()}"
+      output = mix(["run", "-e", inspect, "--", "1", "2", "3"])
+      assert output =~ ~s({:prod, :rpi, ["1", "2", "3"]})
     end)
   after
     System.delete_env("MIX_ENV")
+    System.delete_env("MIX_TARGET")
     System.delete_env("MIX_EXS")
   end
 
-  test "env config defaults to the tasks's preferred cli environment", %{tmp_dir: tmp_dir} do
+  test "cli config", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
+      File.write!("custom.exs", """
+      defmodule P do
+        use Mix.Project
+        def project, do: [app: :p, version: "0.1.0"]
+        def cli, do: [default_env: :prod, default_target: :rpi]
+      end
+      """)
+
+      System.put_env("MIX_EXS", "custom.exs")
+
+      inspect = "IO.inspect {Mix.env(), Mix.target(), System.argv()}"
+      output = mix(["run", "-e", inspect, "--", "1", "2", "3"])
+      assert output =~ ~s({:prod, :rpi, ["1", "2", "3"]})
+    end)
+  after
+    System.delete_env("MIX_EXS")
+  end
+
+  test "preferred cli config", %{tmp_dir: tmp_dir} do
+    File.cd!(tmp_dir, fn ->
+      File.write!("custom.exs", """
+      defmodule P do
+        use Mix.Project
+
+        def cli do
+          [
+            default_env: :not_prod,
+            default_target: :not_rpi,
+            preferred_envs: [test_task: :prod],
+            preferred_targets: [test_task: :rpi]
+          ]
+        end
+
+        def project, do: [app: :p, version: "0.1.0"]
+      end
+
+      defmodule Mix.Tasks.TestTask do
+        use Mix.Task
+
+        def run(args) do
+          IO.inspect {Mix.env(), Mix.target(), args}
+        end
+      end
+      """)
+
+      System.put_env("MIX_EXS", "custom.exs")
+
+      output = mix(["test_task", "a", "b", "c"])
+      assert output =~ ~s({:prod, :rpi, ["a", "b", "c"]})
+    end)
+  after
+    System.delete_env("MIX_EXS")
+  end
+
+  test "env config and target use defaults when empty", %{tmp_dir: tmp_dir} do
     File.cd!(tmp_dir, fn ->
       File.write!("custom.exs", """
       defmodule P do
         use Mix.Project
         def project, do: [app: :p, version: "0.1.0"]
       end
-
-      defmodule Mix.Tasks.TestTask do
-        use Mix.Task
-        @preferred_cli_env :prod
-
-        def run(args) do
-          IO.inspect {Mix.env(), args}
-        end
-      end
       """)
 
+      System.put_env("MIX_ENV", "")
+      System.put_env("MIX_TARGET", "")
       System.put_env("MIX_EXS", "custom.exs")
 
-      output = mix(["test_task", "a", "b", "c"])
-      assert output =~ ~s({:prod, ["a", "b", "c"]})
-    end)
-  after
-    System.delete_env("MIX_EXS")
-  end
-
-  test "target config defaults to the user's preferred cli target", %{tmp_dir: tmp_dir} do
-    File.cd!(tmp_dir, fn ->
-      File.write!("custom.exs", """
-      defmodule P do
-        use Mix.Project
-        def project, do: [app: :p, version: "0.1.0", preferred_cli_target: [test_task: :other]]
-      end
-
-      defmodule Mix.Tasks.TestTask do
-        use Mix.Task
-
-        def run(args) do
-          IO.inspect {Mix.target, args}
-        end
-      end
-      """)
-
-      System.put_env("MIX_EXS", "custom.exs")
-
-      output = mix(["test_task", "a", "b", "c"])
-      assert output =~ ~s({:other, ["a", "b", "c"]})
+      inspect = "IO.inspect {Mix.env(), Mix.target(), System.argv()}"
+      output = mix(["run", "-e", inspect, "--", "a", "b"])
+      assert output =~ ~s({:dev, :host, ["a", "b"]})
     end)
   after
+    System.delete_env("MIX_ENV")
+    System.delete_env("MIX_TARGET")
     System.delete_env("MIX_EXS")
   end
 
