@@ -112,7 +112,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
+      ensure_touched(__ENV__.file, "_build/dev/lib/sample/.mix/compile.elixir")
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
@@ -123,14 +123,13 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       end
       """)
 
+      ensure_touched(__ENV__.file, "_build/dev/lib/sample/.mix/compile.elixir")
       File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
       assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
-      # Making the manifest olds returns :ok, but does not recompile.
-      # Note we use ensure_touched instead of @old_time for preciseness.
       ensure_touched(__ENV__.file, "_build/dev/lib/sample/.mix/compile.elixir")
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
       refute_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
@@ -144,11 +143,6 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       Process.put({MixTest.Case.Sample, :application}, extra_applications: [:logger])
       File.mkdir_p!("config")
 
-      File.write!("config/config.exs", """
-      import Config
-      config :logger, :level, :debug
-      """)
-
       File.write!("lib/a.ex", """
       defmodule A do
         _ = Logger.metadata()
@@ -162,6 +156,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       recompile = fn ->
         Mix.ProjectStack.pop()
         Mix.Project.push(MixTest.Case.Sample)
+        ensure_touched("config/config.exs", "_build/dev/lib/sample/.mix/compile.elixir")
         Mix.Tasks.Loadconfig.load_compile("config/config.exs")
         Mix.Tasks.Compile.Elixir.run(["--verbose"])
       end
@@ -172,11 +167,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :logger, :level, :debug
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Changing config recompiles
       File.write!("config/config.exs", """
@@ -184,25 +177,20 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :logger, :level, :info
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Removing config recompiles
       File.write!("config/config.exs", """
       import Config
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # No-op does not recompile
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       refute_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
@@ -213,11 +201,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :sample, :foo, :bar
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Changing an unknown dependency returns :ok but does not recompile
       File.write!("config/config.exs", """
@@ -225,12 +211,6 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :sample, :foo, :bar
       config :unknown, :unknown, :unknown
       """)
-
-      # We use ensure_touched because an outdated manifest would recompile anyway.
-      ensure_touched("config/config.exs", "_build/dev/lib/sample/.mix/compile.elixir")
-      assert recompile.() == {:ok, []}
-      refute_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
-      refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
     end)
   after
     Application.delete_env(:sample, :foo, persistent: true)
@@ -292,6 +272,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       recompile = fn ->
         Mix.ProjectStack.pop()
         Mix.Project.push(MixTest.Case.Sample)
+        ensure_touched("config/config.exs", "_build/dev/lib/sample/.mix/compile.elixir")
         Mix.Tasks.Loadconfig.load_compile("config/config.exs")
         Mix.Tasks.Compile.Elixir.run(["--verbose"])
       end
@@ -302,11 +283,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :ex_unit, :some, :config
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Changing config recompiles
       File.write!("config/config.exs", """
@@ -314,25 +293,20 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :ex_unit, :some, :another
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Removing config recompiles
       File.write!("config/config.exs", """
       import Config
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # No-op does not recompile
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       refute_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
@@ -361,6 +335,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       recompile = fn ->
         Mix.ProjectStack.pop()
         Mix.Project.push(MixTest.Case.Sample)
+        ensure_touched("config/config.exs", "_build/dev/lib/sample/.mix/compile.elixir")
         Mix.Tasks.Loadconfig.load_compile("config/config.exs")
         Mix.Tasks.Compile.Elixir.run(["--verbose"])
       end
@@ -371,11 +346,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :logger, :compile_time_purge_matching, []
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Inserting a bogus config should crash
       File.write!("config/config.exs", """
@@ -383,7 +356,6 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :logger, :compile_time_purge_matching, [level_lower_than: :debug]
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       ExUnit.CaptureIO.capture_io(:stderr, fn -> assert {:error, _} = recompile.() end)
 
       # Revering the original config should recompile
@@ -392,11 +364,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :logger, :compile_time_purge_matching, []
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
     end)
   after
     Application.put_env(:logger, :compile_time_purge_matching, [])
@@ -420,6 +390,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       recompile = fn ->
         Mix.ProjectStack.pop()
         Mix.Project.push(MixTest.Case.Sample)
+        ensure_touched("config/config.exs", "_build/dev/lib/sample/.mix/compile.elixir")
         Mix.Tasks.Loadconfig.load_compile("config/config.exs")
         Mix.Tasks.Compile.Elixir.run(["--verbose"])
       end
@@ -430,11 +401,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :logger, :level, :debug
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Changing config recompiles
       File.write!("config/config.exs", """
@@ -442,22 +411,18 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :logger, :level, :info
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Removing config recompiles
       File.write!("config/config.exs", """
       import Config
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Changing self fully recompiles
       File.write!("config/config.exs", """
@@ -465,11 +430,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       config :sample, :foo, :bar
       """)
 
-      File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
-      assert File.stat!("_build/dev/lib/sample/.mix/compile.elixir").mtime > @old_time
 
       # Changing an unknown dependency returns :ok but does not recompile
       File.write!("config/config.exs", """
@@ -479,7 +442,6 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       """)
 
       # We use ensure_touched because an outdated manifest would recompile anyway.
-      ensure_touched("config/config.exs", "_build/dev/lib/sample/.mix/compile.elixir")
       assert recompile.() == {:ok, []}
       refute_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
@@ -515,8 +477,8 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       %{"logger": :unused}
       """)
 
+      ensure_touched("mix.lock")
       File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
-      File.touch!("_build/dev/lib/sample/.mix/compile.app_cache", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
@@ -527,6 +489,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       %{"logger": :another}
       """)
 
+      ensure_touched("mix.lock")
       File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
@@ -538,6 +501,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       %{}
       """)
 
+      ensure_touched("mix.lock")
       File.touch!("_build/dev/lib/sample/.mix/compile.elixir", @old_time)
       assert recompile.() == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
@@ -795,7 +759,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
 
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A, B])
 
       same_length_content = "lib/a.ex" |> File.read!() |> String.replace("A", "Z")
@@ -828,7 +792,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
 
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A, B])
 
       future = {{2038, 1, 1}, {0, 0, 0}}
@@ -859,7 +823,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
 
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A, B])
 
       # Compile with error
@@ -898,7 +862,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
 
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A, B])
 
       File.write!("lib/a.ex", File.read!("lib/a.ex") <> "\n")
@@ -919,7 +883,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
 
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A, B])
 
       force_recompilation("lib/b.ex")
@@ -968,7 +932,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
 
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A, B])
 
       File.rm("lib/b.ex")
@@ -1028,15 +992,14 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       # Compiles with missing external resources
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:noop, []}
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A, B])
 
       # Update local existing resource timestamp is not enough
       File.touch!("lib/a.eex", {{2038, 1, 1}, {0, 0, 0}})
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:noop, []}
 
-      File.write!("lib/a.eex", [File.read!("lib/a.eex"), ?\n])
-      File.touch!("lib/a.eex", {{2038, 1, 1}, {0, 0, 0}})
+      force_recompilation("lib/a.eex")
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
@@ -1044,7 +1007,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       # Does not update on old existing resource
       File.touch!("lib/a.eex", @old_time)
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:noop, []}
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A, B])
 
       # Create external resource
@@ -1227,7 +1190,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
       refute function_exported?(A, :one, 0)
 
-      Mix.shell().flush
+      Mix.shell().flush()
       purge([A])
 
       File.rm("lib/b.ex")
@@ -1489,7 +1452,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       Mix.Project.push(MixTest.Case.Sample)
 
       File.write!("lib/a.ex", """
-      IO.warn "warning", [{nil, nil, 0, file: 'lib/foo.txt', line: 3}]
+      IO.warn "warning", [{nil, nil, 0, file: ~c"lib/foo.txt", line: 3}]
       """)
 
       capture_io(:stderr, fn ->
@@ -1634,11 +1597,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
         def foo(), do: B.foo()
         def bar(), do: B.bar()
         def __after_verify__(__MODULE__) do
-          if Code.ensure_loaded?(B) and not function_exported?(B, :foo, 0) do
-            :ok
-          else
-            IO.warn("AFTER_VERIFY", __ENV__)
-          end
+          IO.warn("AFTER_VERIFY", __ENV__)
         end
       end
       """)
@@ -1667,10 +1626,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
         end)
 
       # Check B due to direct dependency on A
-      # Check C due to transient dependency on A
       assert output =~ "A.foo/0 is undefined or private"
-      assert output =~ "B.bar/0 is undefined or private"
-      assert output =~ "AFTER_VERIFY"
+      refute output =~ "B.bar/0 is undefined or private"
+      refute output =~ "AFTER_VERIFY"
 
       # Ensure only A was recompiled
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
@@ -1687,7 +1645,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert output =~ "B.bar/0 is undefined or private"
       assert output =~ "AFTER_VERIFY"
 
-      # Now we change B and it must no longer emit an AFTER_VERIFY warning
+      # Now we change B and it must emit an AFTER_VERIFY warning
       File.write!("lib/b.ex", """
       defmodule B do
       end
@@ -1700,7 +1658,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
 
       assert output =~ "B.foo/0 is undefined or private"
       assert output =~ "B.bar/0 is undefined or private"
-      refute output =~ "AFTER_VERIFY"
+      assert output =~ "AFTER_VERIFY"
     end)
   end
 
