@@ -40,11 +40,21 @@ defmodule Mix.Tasks.TestTest do
       assert ex_unit_opts([]) == [
                autorun: false,
                exit_status: 2,
-               failures_manifest_file:
+               failures_manifest_path:
                  Path.join(Mix.Project.manifest_path(), ".mix_test_failures")
              ]
     end
 
+    test "respect failures_manifest_path option" do
+      custom_manifest_file = Path.join(Mix.Project.manifest_path(), ".mix_test_failures_custom")
+
+      assert ex_unit_opts(failures_manifest_path: custom_manifest_file) == [
+               autorun: false,
+               exit_status: 2,
+               failures_manifest_path: custom_manifest_file
+             ]
+    end
+
     defp ex_unit_opts(opts) do
       {ex_unit_opts, _allowed_files} = Mix.Tasks.Test.process_ex_unit_opts(opts)
       ex_unit_opts
@@ -53,7 +63,7 @@ defmodule Mix.Tasks.TestTest do
     defp ex_unit_opts_from_given(passed) do
       passed
       |> ex_unit_opts()
-      |> Keyword.drop([:failures_manifest_file, :autorun, :exit_status])
+      |> Keyword.drop([:failures_manifest_path, :autorun, :exit_status])
     end
   end
 
@@ -265,7 +275,7 @@ defmodule Mix.Tasks.TestTest do
       in_fixture("test_stale", fn ->
         port = mix_port(~w[test --stale --listen-on-stdin])
 
-        assert receive_until_match(port, "seed", "") =~ "2 tests"
+        assert receive_until_match(port, "0 failures", "") =~ "2 tests"
 
         Port.command(port, "\n")
 
@@ -294,7 +304,7 @@ defmodule Mix.Tasks.TestTest do
 
         Port.command(port, "\n")
 
-        assert receive_until_match(port, "seed", "") =~ "2 tests"
+        assert receive_until_match(port, "0 failures", "") =~ "2 tests"
 
         File.write!("test/b_test_stale.exs", """
         defmodule BTest do
@@ -323,7 +333,7 @@ defmodule Mix.Tasks.TestTest do
 
         Port.command(port, "\n")
 
-        assert receive_until_match(port, "seed", "") =~ "2 tests"
+        assert receive_until_match(port, "0 failures", "") =~ "2 tests"
       end)
     end
   end
@@ -477,38 +487,33 @@ defmodule Mix.Tasks.TestTest do
 
         output = mix(["test", "apps/bar/test/bar_tests.exs"])
 
-        assert output =~ """
-               ==> bar
-               ....
-               """
+        assert output =~ "==> bar"
+        assert output =~ "...."
 
         refute output =~ "==> foo"
         refute output =~ "Paths given to \"mix test\" did not match any directory/file"
 
         output = mix(["test", "./apps/bar/test/bar_tests.exs"])
 
-        assert output =~ """
-               ==> bar
-               ....
-               """
+        assert output =~ "==> bar"
+        assert output =~ "...."
 
         refute output =~ "==> foo"
         refute output =~ "Paths given to \"mix test\" did not match any directory/file"
 
         output = mix(["test", Path.expand("apps/bar/test/bar_tests.exs")])
 
-        assert output =~ """
-               ==> bar
-               ....
-               """
+        assert output =~ "==> bar"
+        assert output =~ "...."
 
         refute output =~ "==> foo"
         refute output =~ "Paths given to \"mix test\" did not match any directory/file"
 
         output = mix(["test", "apps/bar/test/bar_tests.exs:10"])
 
+        assert output =~ "==> bar"
+
         assert output =~ """
-               ==> bar
                Excluding tags: [:test]
                Including tags: [location: {"test/bar_tests.exs", 10}]
 
