@@ -75,7 +75,7 @@ defmodule Mix.Tasks.XrefTest do
 
         Mix.Task.run("compile")
         after_compile.()
-        xref = Mix.Tasks.Xref
+        xref = String.to_atom("Elixir.Mix.Tasks.Xref")
         assert Enum.sort(xref.calls()) == Enum.sort(expected)
       end)
     end
@@ -445,9 +445,9 @@ defmodule Mix.Tasks.XrefTest do
 
       Cycle of length 3:
 
-          lib/b.ex
           lib/a.ex
           lib/b.ex
+          lib/a.ex
 
       """)
     end
@@ -1085,7 +1085,7 @@ defmodule Mix.Tasks.XrefTest do
                  ^first_line | ["Generated sample app" | result]
                ] = receive_until_no_messages([]) |> String.split("\n")
 
-        assert normalize_graph_output(result |> Enum.join("\n")) == expected
+        assert result |> Enum.join("\n") |> normalize_graph_output() == expected
       end)
     end
 
