@@ -9,6 +9,10 @@ System.put_env("MIX_HOME", mix)
 System.delete_env("XDG_DATA_HOME")
 System.delete_env("XDG_CONFIG_HOME")
 
+# Load protocols to make sure they are not unloaded during tests
+[Collectable, Enumerable, Inspect, String.Chars, List.Chars]
+|> Enum.each(& &1.__protocol__(:module))
+
 ## Setup Mix
 
 Mix.start()
@@ -22,7 +26,13 @@ Application.put_env(:logger, :backends, [])
 
 os_exclude = if match?({:win32, _}, :os.type()), do: [unix: true], else: [windows: true]
 epmd_exclude = if match?({:win32, _}, :os.type()), do: [epmd: true], else: []
-git_exclude = if Mix.SCM.Git.git_version() <= {1, 7, 4}, do: [git_sparse: true], else: []
+
+git_exclude =
+  Mix.SCM.Git.unsupported_options()
+  |> Enum.map(fn
+    :sparse -> {:git_sparse, true}
+    :depth -> {:git_depth, true}
+  end)
 
 {line_exclude, line_include} =
   if line = System.get_env("LINE"), do: {[:test], [line: line]}, else: {[], []}
